unit ProcessHook;

// written by @L.CHEMIST (Andrey A. Meshkov)
// http://maalchemist.narod.ru
// maalchemist@yandex.ru
// maalchemist@gmail.com

interface

uses
  Windows, Classes, SysUtils, acRZDriverTypes, acRZDriver;

const
  CResult : array [Boolean] of string = ('(-)', '(+)');

type
  P_ProcessHookData = ^T_ProcessHookData;
  T_ProcessHookData = record
    ParentID  : Int64; // parent process ID
    ProcessID : Int64; // process ID
    Create    : Int64; // create state
    Result    : Int64; // result
  end;

  P_ProcessHookResult = ^T_ProcessHookResult;
  T_ProcessHookResult = packed record
    CP_RecvCount : DWORD;
    CP_BuffDepth : WORD;
    CP_DropCount : WORD;
  end;

type
  T_ProcessHookThread = class (TThread)
  private
    FTermEvent    : THandle;
    FProcessEvent : THandle;
  protected
    procedure Execute; override;
  public
    procedure SET_();
    procedure RET_();
    procedure GET_();

    constructor Create; reintroduce; virtual;
    destructor Destroy; override;

    procedure message_(const S: string);
  end;

procedure CreateProcessHookThread;
procedure CloseProcessHookThread;

procedure ProcessHook_SET;
procedure ProcessHook_RET;

implementation

uses
  TESTmain, TESTUtils;

var
  zProcessHookThread : T_ProcessHookThread = nil;

{
}
procedure CreateProcessHookThread;
begin
  if RZDriver.handle = INVALID_HANDLE_VALUE then Exit;

  if zProcessHookThread <> nil then Exit;
  zProcessHookThread := T_ProcessHookThread.Create;
  zProcessHookThread.Suspended := false;
end;

{
}
procedure CloseProcessHookThread;
begin
  if zProcessHookThread = nil then Exit;

  try
    zProcessHookThread.RET_();
    zProcessHookThread.Terminate;
    SetEvent (zProcessHookThread.FTermEvent);
  except
  end;
end;

{
}
procedure ProcessHook_SET;
begin
  try
    if zProcessHookThread <> nil then zProcessHookThread.SET_();
  except
  end;
end;

{
}
procedure ProcessHook_RET;
begin
  try
    if zProcessHookThread <> nil then zProcessHookThread.RET_();
  except
  end;
end;

{
}
constructor T_ProcessHookThread.Create;
var
  R : Boolean;
  S : string;
begin
  inherited Create (true);

  FreeOnTerminate := true;
  
  FTermEvent := CreateEvent (nil, true, false, nil);

  S := 'Global\' + RZDriver.drvname + '.Event.CreateProcess';
  FProcessEvent := OpenEvent (Windows.SYNCHRONIZE, false, PChar(S));
  R := FProcessEvent <> 0;
  S := CResult [R] + ' ' + S;
  if not R then S := S + ' :: ' + LastErrorMessage;
  message_(S);
end;

{
}
destructor T_ProcessHookThread.Destroy;
begin
  try
    try
      RET_();
      CloseHandle (FProcessEvent);
      CloseHandle (FTermEvent);
    finally
      inherited Destroy;
    end;
  finally
    zProcessHookThread := nil;
  end;
end;

{
}
procedure T_ProcessHookThread.message_(const S: string);
var
  P : P_message;
  L : Integer;
  Z : Integer;
begin
  if TestFormHandle = 0 then Exit;

  L := Length (S);
  if L <= 0 then Exit;

  P := GetMemory (SizeOf (T_message));
  Z := L * SizeOf(Char);
  P.S := GetMemory (Z);
  CopyMemory (P.S, PChar(S), Z);
  P.L := L;

  PostMessage (TestFormHandle, WM_APP_MESSAGE, Integer(P), 0);
end;

{
}
procedure T_ProcessHookThread.SET_();
var
  Q : TRZDriverCPQuery;
begin
  RZDriver_IOCTRL_(RZ_DRIVER_QUERY_CP_SET, Q);
end;

{
}
procedure T_ProcessHookThread.RET_();
var
  Q : TRZDriverCPQuery;
begin
  RZDriver_IOCTRL_(RZ_DRIVER_QUERY_CP_RET, Q);
end;

{
}
procedure T_ProcessHookThread.GET_();
var
  Q : TRZDriverCPQueries;
  P : PRZDriverCPQueries;
  C : DWORD;
  Z : DWORD;
begin
  RZDriver_IOCTRL_(RZ_DRIVER_QUERY_CP_GET, Q);

  C := TRZDriverCPQueryResult (Q[0].Result).CP_BuffDepth;
  if C = 0 then Exit;
  if C > RZ_DRIVER_BUFFER_CAPACITY then Exit;

  Z := C * SizeOf (TRZDriverCPQuery);
  P := GetMemory (Z);
  CopyMemory (P, @Q, Z);
  PostMessage (TestFormHandle, WM_APP_EVENT_CREATE_PROCESS, Integer(P), C);
end;
(*
var
  Q : TRZDriverCPQueries;
  P : P_ProcessHookData;
  C : DWORD;
  I : Integer;
  R : Int64;
begin
  if TestFormHandle = 0 then Exit;
  RZDriver_IOCTRL_(RZ_DRIVER_QUERY_CP_GET, Q);

  for I := 0 to RZ_DRIVER_BUFFER_CAPACITY - 1 do begin
    R := Q[I].Result;
    C := T_ProcessHookResult(R).CP_BuffDepth;
    if C = 0 then Exit;

    P := GetMemory (SizeOf (T_ProcessHookData));
    P.ParentID := Q[I].ParentID;
    P.ProcessID := Q[I].ProcessID;
    P.Create := Q[I].Create;
    P.Result := R;

    PostMessage (TestFormHandle, WM_APP_EVENT_CREATE_PROCESS, Integer(P), 0);
  end;
end; // *)

{
}
procedure T_ProcessHookThread.Execute;
var
  R     : DWORD;
  S     : string;
  VOBJs : array [0..1] of THandle;
begin
  try
    try
      VOBJs[0] := FTermEvent;
      VOBJs[1] := FProcessEvent;

      while not Terminated do begin
        R := WaitForMultipleObjects (2, @VOBJs, false, INFINITE);
        
        if Terminated then Exit;
        if R = WAIT_OBJECT_0 then Exit;
        if R = WAIT_OBJECT_0 + 1 then GET_();
      end;
    except
      on E : Exception do begin
        S := 'T_ProcessHookThread.Execute.Exception :: ' + E.message;
        message_(S);
      end;
    end;
  finally
    Terminate;
  end;
end;

end.

