unit TESTUtils;

// written by @L.CHEMIST (Andrey A. Meshkov)
// http://maalchemist.narod.ru
// maalchemist@yandex.ru
// maalchemist@gmail.com

interface

uses
  Windows;

const
  {$IFDEF UNICODE}
  AWSuffix = 'W';
  {$ELSE}
  AWSuffix = 'A';
  {$ENDIF UNICODE}

const
  SE_DEBUG_NAME = 'SeDebugPrivilege';

  STATUS_SUCCESS = 0;

type
  USHORT = WORD;

  UNICODE_STRING = packed record
    Length        : USHORT;
    MaximumLength : USHORT;
    Buffer        : PWideString;
  end;

  PROCESS_BASIC_INFORMATION = packed record
    ExitStatus       : DWORD;
    PebBaseAddress   : DWORD;
    AffinityMask     : DWORD;
    BasePriority     : DWORD;
    uUniqueProcessId : DWORD;
    uInheritedFromUniqueProcessId : DWORD;
  end;

function LastErrorMessage (AError: DWORD = 0): string;

function EnablePrivilege (const SPrivilege: string; const AEnable: BOOL = true): BOOL;
function GetLongImagePath (const AImage: string): string;
function GetProcessImagePath (AID: DWORD): string;
function GetProcessCommandLine (AID: DWORD): string;
function HasReadAccess (AProcess: THandle; AAddress: Pointer): Boolean; overload;
function ReadProcessUnicodeMemory (AProcess: THandle; AAddress: PChar): string;

implementation

uses
  SysUtils, PsAPI;

type
  TGetLongPathName = function (lpszShortPath: PChar; lpszLongPath: PChar; cchBuffer: DWORD): DWORD; stdcall;
  TGetProcessImageFileName = function (hProcess: THandle; lpImageFileName: PChar; nSize: DWORD): DWORD; stdcall;
  TNtQueryInformationProcess = function (ProcessHandle: THandle; ProcessInformationClass: Integer; ProcessInformation: Pointer; ProcessInformationLength: ULONG; ReturnLength: PULONG): DWORD; stdcall;

var
  HKERNEL : THandle = 0;
  HPSAPI  : THandle = 0;
  HNTDLL  : THandle = 0;

  KERNEL32_GetLongPathName : TGetLongPathName = nil;
  PSAPI_GetProcessImageFileName : TGetProcessImageFileName = nil;
  NTDLL_NtQueryInformationProcess : TNtQueryInformationProcess = nil;

{
}
function LastErrorMessage (AError: DWORD = 0): string;
var
  E : DWORD;
begin
  if AError <> 0 then E := AError else E := GetLastError;
  Result := SysErrorMessage (E);
end;

{
}
function GetLongPathName (lpszShortPath: PChar; lpszLongPath: PChar; cchBuffer: DWORD): DWORD; stdcall;
begin
  Result := 0;
  if @KERNEL32_GetLongPathName <> nil then begin
    Result := KERNEL32_GetLongPathName (lpszShortPath, lpszLongPath, cchBuffer);
  end;
end;

{
}
function GetProcessImageFileName (hProcess: THandle; lpImageFileName: PChar; nSize: DWORD): DWORD; stdcall;
begin
  Result := 0;
  if @PSAPI_GetProcessImageFileName <> nil then begin
    Result := PSAPI_GetProcessImageFileName (hProcess, lpImageFileName, nSize);
  end;
end;

{
}
function NtQueryInformationProcess (ProcessHandle: THandle; ProcessInformationClass: Integer; ProcessInformation: Pointer; ProcessInformationLength: ULONG; ReturnLength: PULONG): DWORD; stdcall;
begin
  Result := ERROR_INVALID_FUNCTION;
  if @NTDLL_NtQueryInformationProcess <> nil then begin
    Result := NTDLL_NtQueryInformationProcess (ProcessHandle, ProcessInformationClass, ProcessInformation, ProcessInformationLength, ReturnLength);
  end;
end;

{
}
function EnablePrivilege (const SPrivilege: string; const AEnable: BOOL = true): BOOL;
const
  CAttributes : array [Boolean] of DWORD = (0, SE_PRIVILEGE_ENABLED);
var
  VToken     : THandle;
  VLUID      : Int64;
  VState     : TTokenPrivileges;
  VPrevState : TTokenPrivileges;
  VSize      : DWORD;
  VLength    : DWORD;
  VPrivs     : TPrivilegeSet;
  VEnabled   : BOOL;
begin
  Result := false;

  if not OpenProcessToken (GetCurrentProcess, TOKEN_ADJUST_PRIVILEGES or TOKEN_QUERY, VToken) then Exit;
  if not LookupPrivilegeValue (nil, PChar(SPrivilege), VLUID) then Exit;

  try
    VPrivs.PrivilegeCount := 1;
    VPrivs.Control := PRIVILEGE_SET_ALL_NECESSARY;
    VPrivs.Privilege[0].Luid := VLUID;
    VPrivs.Privilege[0].Attributes := 0;
    if PrivilegeCheck (VToken, VPrivs, VEnabled) then begin
      if AEnable = VEnabled then Exit;
    end;

    VSize := SizeOf (TTokenPrivileges);
    VLength := 0;
    VState.PrivilegeCount := 1;
    VState.Privileges[0].Luid := VLUID;
    VState.Privileges[0].Attributes := CAttributes [AEnable];

    Result := AdjustTokenPrivileges (VToken, false, VState, VSize, VPrevState, VLength);
  finally
    CloseHandle (VToken);
  end;
end;

{
}
function GetLongImagePath (const AImage: string): string;
var
  L : Integer;
  S : string;
begin
  Result := AImage;

  L := GetLongPathName (PChar(AImage), PChar(S), 0);
  if L > 0 then begin
    SetLength (S, L + 1);
    L := GetLongPathName (PChar(AImage), PChar(S), L);
    if L > 0 then SetString (Result, PChar(S), L);
  end;
end;

{
  !!! - ,  GetProcessImagePath
           SE_DEBUG_NAME ('SeDebugPrivilege')
  x86 & x64
}
function GetProcessImagePath (AID: DWORD): string;

  { FIX_() }
  function FIX_(const S: string): string;
  var
    L        : Integer;
    LD       : Integer;
    SDrive   : string;
    SDevice  : string;
    SProcess : string;
    VDrives  : string;
    PDrives  : PChar;
    VDevice  : string;
    PDevice  : PChar;
  begin
    Result := StrPas(PChar(S));

    SetLength (VDrives, 1024);
    SetLength (VDevice, 1024);

    PDrives := PChar (VDrives);
    if GetLogicalDriveStrings (1024, PDrives) > 0 then begin
      SProcess := Result;

      L := lstrlen (PDrives);
      while L > 0 do begin
        SetString (SDrive, PDrives, L);
        {$IFDEF XE2}
        if CharInSet (SDrive[L], ['\', '/']) then Delete (SDrive, L, 1);
        {$ELSE}
        if SDrive[L] in ['\', '/'] then Delete (SDrive, L, 1);
        {$ENDIF}

        PDevice := PChar (VDevice);
        if QueryDosDevice (PChar(SDrive), PDevice, Length(VDevice)) > 0 then begin
          LD := lstrlen (PDevice);
          if LD > 0 then begin
            SetString (SDevice, PDevice, LD);
            if AnsiPos (SDevice, SProcess) = 1 then begin
              Delete (SProcess, 1, Length(SDevice));
              if Length (SProcess) > 0 then begin
                {$IFDEF XE2}
                if CharInSet (SProcess[1], ['\', '/']) then Delete (SProcess, 1, 1);
                {$ELSE}
                if SProcess[1] in ['\', '/'] then Delete (SProcess, 1, 1);
                {$ENDIF}
                Result := SDrive + '\' + SProcess;
              end;

              Break;
            end;
          end;
        end;

        PDrives := PDrives + L + 1;
        L := lstrlen (PDrives);
      end;
    end;
  end;

  { GET_module_() }
  function GET_module_(AID: DWORD): string;
  var
    VProcess : THandle;
    SProcess : string;
    PBuffer  : PChar;
    VBuffer  : string;
  begin
    Result := '';

    VProcess := OpenProcess (PROCESS_QUERY_INFORMATION or PROCESS_VM_READ, false, AID);
    if VProcess = 0 then Exit;

    try
      SetLength (VBuffer, 1024);
      PBuffer := PChar(VBuffer);
      if GetModuleFilenameEx (VProcess, 0, PBuffer, Length(VBuffer)) > 0 then begin
        SProcess := StrPas (PBuffer);
        Result := GetLongImagePath (SProcess);
      end;
    finally
      CloseHandle (VProcess);
    end;
  end;

  { GET_x86_() }
  function GET_x86_(AID: DWORD): string;
  var
    VProcess : THandle;
    SProcess : string;
    VModule  : THandle;
    PBuffer  : PChar;
    VBuffer  : string;
    VBytes   : DWORD;
  begin
    Result := '';

    VProcess := OpenProcess (PROCESS_QUERY_INFORMATION or PROCESS_VM_READ, false, AID);
    if VProcess = 0 then Exit;

    try
      if EnumProcessModules (VProcess, @VModule, 4, VBytes) then begin
        SetLength (VBuffer, 1024);
        PBuffer := PChar(VBuffer);
        if GetModuleFilenameEx (VProcess, VModule, PBuffer, Length(VBuffer)) > 0 then begin
          SProcess := StrPas (PBuffer);
          Result := GetLongImagePath (SProcess);
        end;
      end;
    finally
      CloseHandle (VProcess);
    end;
  end;

  { GET_x64_() }
  function GET_x64_(AID: DWORD): string;
  var
    L        : DWORD;
    VProcess : THandle;
    SProcess : string;
    VPImage  : string;
    PPImage  : PChar;
  begin
    Result := '';

    VProcess := OpenProcess (PROCESS_QUERY_INFORMATION or PROCESS_VM_READ, false, AID);
    if VProcess = 0 then Exit;

    try
      L := 1024;
      SetLength (VPImage, L);
      ZeroMemory (PChar(VPImage), L * SizeOf(Char));
      PPImage := PChar(VPImage);

      L := GetProcessImageFileName (VProcess, PPImage, L);
      if L > 0 then begin
        SProcess := FIX_(VPImage);
        Result := GetLongImagePath (SProcess);
      end;
    finally
      CloseHandle (VProcess);
    end;
  end;

  { GET_vm_cmd_() }
  function GET_vm_cmd_(AID: DWORD): string;
  begin
    Result := GetProcessCommandLine (AID);
  end;

  { OK_() }
  function OK_(const S: string): Boolean;
  begin
    Result := (AnsiPos(':', S) > 0) and FileExists (S);
  end;

var
  S : string;
begin
  if AID = 0 then begin
    Result := 'System Idle Process';
    Exit;
  end;

  if AID in [2, 4, 8] then begin
    Result := 'System';
    Exit;
  end;

  Result := GET_module_(AID);
  if OK_(Result) then Exit;

  S := GET_x86_(AID);
  if S <> '' then begin
    Result := S;
    if OK_(Result) then Exit;
  end;

  S := GET_x64_(AID);
  if S <> '' then begin
    Result := S;
    if OK_(Result) then Exit;
  end;

  S := GET_vm_cmd_(AID);
  if S <> '' then begin
    Result := S;
    if OK_(Result) then Exit;
  end;
end;

{
}
function GetProcessCommandLine (AID: DWORD): string;
var
  VSize    : DWORD;
  VProcess : DWORD;
  {$IFDEF XE2}
  VBytes   : NativeUInt;
  {$ELSE}
  VBytes   : DWORD;
  {$ENDIF}
  VInfo    : PROCESS_BASIC_INFORMATION;
  VParams  : PChar;
  VData    : PChar;
  VPriSet  : BOOL;
begin
  Result := '';

  VPriSet := EnablePrivilege (SE_DEBUG_NAME);
  try
    VProcess := OpenProcess (PROCESS_QUERY_INFORMATION or PROCESS_VM_READ, false, AID);
    if VProcess = 0 then Exit;

    try
      VSize := SizeOf (VInfo);
      FillChar (VInfo, VSize, 0);
      if STATUS_SUCCESS <> NtQueryInformationProcess (VProcess, 0, @VInfo, VSize, nil) then Exit;

      if not ReadProcessMemory (VProcess, PChar(VInfo.PebBaseAddress) + $10,
        @VParams, SizeOf (VParams), VBytes) then Exit;

      if not ReadProcessMemory (VProcess, VParams + $44,
        @VData, SizeOf (VData), VBytes) then Exit;

      Result := PChar (ReadProcessUnicodeMemory (VProcess, VData));
    finally
      CloseHandle (VProcess);
    end;
  finally
    if VPriSet then EnablePrivilege (SE_DEBUG_NAME, false);
  end;
end;

{
}
function HasReadAccess (AProcess: THandle; AAddress: Pointer): Boolean; overload;
var
	Vmemory : MEMORY_BASIC_INFORMATION;
begin
  Result := false;

  try
    VirtualQueryEx (AProcess, AAddress, Vmemory, SizeOf(Vmemory));
    if (PAGE_NOACCESS = Vmemory.Protect) or (PAGE_EXECUTE = Vmemory.Protect) then Exit;

    Result := true;
  except
  end;
end;

{
}
function ReadProcessUnicodeMemory (AProcess: THandle; AAddress: PChar): string;
var
  S : WideString;
  C : WideChar;
  I : DWORD;
  Z : DWORD;
  {$IFDEF XE2}
  B : NativeUInt;
  {$ELSE}
  B : DWORD;
  {$ENDIF}
begin
  Result := '';
  try
    if not HasReadAccess (AProcess, AAddress) then Exit;

    S := '';
    I := 0;
    Z := 2;
    while ReadProcessMemory (AProcess, AAddress + I, @C, Z, B) do begin
      S := S + C;
      Inc (I, Z);
    end;
    Result := WideCharLenToString (PWideChar(S), Length(S));
  except
  end;
end;

{
}
procedure Init;
begin
  HKERNEL := LoadLibrary ('KERNEL32.DLL');
  if HKERNEL < 32 then HKERNEL := 0;

  HPSAPI := LoadLibrary ('PSAPI.DLL');
  if HPSAPI < 32 then HPSAPI := 0;

  HNTDLL := LoadLibrary ('NTDLL.DLL');
  if HNTDLL < 32 then HNTDLL := 0;

  if HKERNEL <> 0 then begin
    @KERNEL32_GetLongPathName := GetProcAddress (HKERNEL, 'GetLongPathName' + AWSuffix);
  end;

  if HPSAPI <> 0 then begin
    @PSAPI_GetProcessImageFileName := GetProcAddress (HPSAPI, 'GetProcessImageFileName' + AWSuffix);
  end;

  if HNTDLL <> 0 then begin
    @NTDLL_NtQueryInformationProcess := GetProcAddress (HNTDLL, 'NtQueryInformationProcess');
  end;
end;

{
}
procedure Done;

  procedure _ (var ALibrary: THandle);
  var
    H : THandle;
  begin
    try
      if ALibrary <> 0 then begin
        H := ALibrary;
        ALibrary := 0;
        FreeLibrary (H);
      end;
    except
    end;
  end;

begin
  _ (HNTDLL);
  _ (HKERNEL);
  _ (HPSAPI);
end;

initialization
  TESTUtils.Init;

finalization
  TESTUtils.Done;

end.

