unit TESTmain;

// written by @L.CHEMIST (Andrey A. Meshkov)
// http://maalchemist.narod.ru
// maalchemist@yandex.ru
// maalchemist@gmail.com

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;

const
  WM_APP_MESSAGE              = WM_APP + 1;
  WM_APP_EVENT_CREATE_PROCESS = WM_APP + 2;

type
  P_message = ^T_message;
  T_message = record
    S : PChar;
    L : Integer;
  end;
  
type
  TTestForm = class(TForm)
    gbOperation: TGroupBox;
    GB_Msgs: TGroupBox;
    ED_Msgs: TMemo;
    GB_Call: TGroupBox;
    BT_noargs: TButton;
    BT_regcall: TButton;
    BT_stdcall: TButton;
    BT_cdecl: TButton;
    GB_Port: TGroupBox;
    L_Freq: TLabel;
    L_Delay: TLabel;
    ED_Freq: TEdit;
    ED_Delay: TEdit;
    BT_PlaySound: TButton;
    BT_BIOSDump: TButton;
    GB_ProcessHook: TGroupBox;
    BT_ProcessHookSet: TButton;
    BT_ProcessHookRet: TButton;
    BT_Beep: TButton;
    BT_fastcall: TButton;

    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure BT_noargsClick(Sender: TObject);
    procedure BT_regcallClick(Sender: TObject);
    procedure BT_stdcallClick(Sender: TObject);
    procedure BT_cdeclClick(Sender: TObject);
    procedure BT_PlaySoundClick(Sender: TObject);
    procedure BT_BIOSDumpClick(Sender: TObject);
    procedure BT_ProcessHookSetClick(Sender: TObject);
    procedure BT_ProcessHookRetClick(Sender: TObject);
    procedure BT_BeepClick(Sender: TObject);
    procedure BT_fastcallClick(Sender: TObject);
  private
    procedure WMAppMessage (var message: TMessage); message WM_APP_MESSAGE;
    procedure WMAppEventCreateProcess (var message: TMessage); message WM_APP_EVENT_CREATE_PROCESS;
  public
    procedure message_(const S: string);
    procedure Refreshh (ASender: TObject);
  end;

var
  TestForm : TTestForm = nil;
  TestFormHandle : THandle = 0;

implementation

{$R *.dfm}

uses
  ProcessHook, TESTUtils, acRZDriver, acRZDriverTypes, acDriverUtils, acRZIO,
  acRZBeep;

{
}
procedure TTestForm.message_(const S: string);
begin
  ED_Msgs.Lines.ADD (S);
end;

{
}
procedure TTestForm.Refreshh (ASender: TObject);
begin
  if ASender is TButton then TButton(ASender).Refresh;
  Application.ProcessMessages;
end;

{
}
procedure TTestForm.FormCreate(Sender: TObject);
const
  Cx64 : array [Boolean] of string = ('x86', 'x64');
var
  SD  : string;
  VR  : Boolean;
  VRC : Integer;
begin
  TestFormHandle := Handle;
  VR := acRZDriver.RZDriver_Open ('', '', VRC);

  SD := '(' + Cx64 [RZDriver.x64] + ') ' + RZDriver.drvname;
  if VR then begin
    message_('OPEN RZ DRIVER ' + SD);
  end else begin
    if VRC = -1 then begin
      message_('RZ DRIVER ALREADY OPENED ' + SD);
    end else begin
      message_('ERROR: ' + LastErrorMessage (acDriverUtils.LastError));
    end;
  end;

  CreateProcessHookThread;
end;

{
}
procedure TTestForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  CloseProcessHookThread;
  TestFormHandle := 0;

  Action := caFree;
end;

{
}
procedure TTestForm.FormDestroy(Sender: TObject);
begin
  acRZDriver.RZDriver_Close;
end;

{
}
procedure TTestForm.WMAppMessage (var message: TMessage); // message WM_APP_MESSAGE;
var
  P : P_message;
  S : string;
begin
  inherited;

  P := P_message (message.WParam);
  if P = nil then Exit;

  try
    if P.S <> nil then try
      if P.L > 0 then begin
        SetString (S, P.S, P.L);
      end;
    finally
      FreeMemory (P.S);
    end;
  finally
    FreeMemory (P);
  end;

  message_(S);
end;

{
}
procedure TTestForm.WMAppEventCreateProcess (var message: TMessage); // message WM_APP_EVENT_CREATE_PROCESS;
const
  CResult : array [Boolean] of string = ('(-)', '(+)');
var
  Q             : TRZDriverCPQueries;
  P             : PRZDriverCPQueries;
  C             : DWORD;
  Z             : DWORD;
  I             : Integer;
  S             : string;
  V             : string;

  VParentID     : DWORD;
  VProcessID    : DWORD;
  ICreate       : BYTE;
  VCreate       : Boolean;
  VResult       : Int64;

  VRZRecvCount  : DWORD;
  VRZBuffDepth  : DWORD;
  VRZDropCount  : DWORD;
begin
  inherited;

  P := PRZDriverCPQueries (message.WParam);
  if P = nil then Exit;

  C := message.LParam;
  if C = 0 then Exit;
  if C > RZ_DRIVER_BUFFER_CAPACITY then Exit;

  try
    Z := C * SizeOf (TRZDriverCPQuery);
    CopyMemory (@Q, P, Z);
  finally
    FreeMemory (P);
  end;

  for I := 0 to C - 1 do begin
    VParentID := Q[I].ParentID;
    VProcessID := Q[I].ProcessID;
    ICreate := Q[I].Create;
    VCreate := Boolean (ICreate);
    VResult := Q[I].Result;

    VRZRecvCount := TRZDriverCPQueryResult (VResult).CP_RecvCount;
    VRZBuffDepth := TRZDriverCPQueryResult (VResult).CP_BuffDepth;
    VRZDropCount := TRZDriverCPQueryResult (VResult).CP_DropCount;

    S := Format ('%s PROCESS (R:%u/%u/%u) %u << %u', [
      CResult [VCreate], VRZDropCount, VRZBuffDepth, VRZRecvCount, VProcessID, VParentID]);
    V := GetProcessImagePath (VProcessID);
    if V <> '' then S := S + ' >> ' + V;

    message_(S);
  end;
end;
(*
var
  P : P_ProcessHookData;
  S : string;
  V : string;
begin
  inherited;

  P := P_ProcessHookData (message.WParam);
  if P = nil then Exit;

  try
    if P.Create <> 0 then begin
      S := '(+) Process ';
    end else begin
      S := '(-) Process ';
    end;
    S := S + IntToStr (P.ProcessID) + ' << ' + IntToStr (P.ParentID);
    V := GetProcessImagePath (P.ProcessID);
    if V <> '' then S := S + ' >> ' + V;
  finally
    FreeMemory (P);
  end;

  message_(S);
end; // *)

{
}
function ReadTSC: Int64;
asm
        dw      $310F // for Delphi 4,5 compatibility
//      rdtsc
end;

{
}
procedure base_beep; register;
asm
        mov     al, 0B6h
        out     43h, al
        in      al, 61h
        or      al, 3
        out     61h, al

        mov     ax, 600
        mov     ecx, 30000
    @MUSIC:
        {$IFDEF CPUX64}
        push    rcx
        {$ELSE}
        push    ecx
        {$ENDIF}

        out     42h, al
        mov     al, ah
        out     42h, al

        mov     ecx, 3000
    @DELAY:
        sub     ecx, 1 // dec ecx : ??? don`t work in some cases
        jnz     @DELAY
    @DELAYOUT:
        {$IFDEF CPUX64}
        pop     rcx
        {$ELSE}
        pop     ecx
        {$ENDIF}
        sub     ecx, 1 // dec ecx : ??? don`t work in some cases
        jnz     @MUSIC

        and     al, 11111100b
        out     61h, al
end;

{
}
procedure base_beeps; register;
asm
        cli
        // speaker ON
        in      al, 61h
        or      al, 11b
        out     61h, al
        sti

        mov     ecx, 18000000h
    @DELAY:
        sub     ecx, 1 // dec ecx : ??? don`t work in some cases
        jnz     @DELAY

        cli
        // speaker OFF
        in      al, 61h
        and     al, 11111100b
        out     61h, al
        sti
end;

{
  call user-mode procedure with no arguments from ring-0
}
procedure TTestForm.BT_noargsClick(Sender: TObject);
var
  Q : TRZDriverQuery;
begin
  Refreshh (Sender);

  FillChar (Q, SizeOf(Q), 0);
  Q.IOCODE := RZ_DRIVER_QUERY_PROC_NOARGS;
  Q.PROC   := Int64(@base_beep);

  if RZDriver_IOCTRL (Q) then begin
    message_('SUCCESS');
  end else begin
    message_('ERROR: ' + LastErrorMessage);
  end;
end;

{
  -> EAX = AParam
}
procedure register_beep (AParam: Pointer); register;
{$IFDEF CPUX64}
asm
        // save argument
        push    rax

        call    base_beep

        // do something with argument
        pop     rax
        mov     edx, [rax]
        add     edx, 1 // inc edx : ??? don`t work in some cases
        mov     [rax], edx
end;
{$ELSE}
asm
        // save argument
        push    eax

        call    base_beep

        // do something with argument
        pop     eax
        mov     edx, [eax]
        add     edx, 1 // inc edx : ??? don`t work in some cases
        mov     [eax], edx
end;
{$ENDIF}

{
  call user-mode register procedure from ring-0
}
procedure TTestForm.BT_regcallClick(Sender: TObject);
var
  P : DWORD;
  Q : TRZDriverQuery;
begin
  Refreshh (Sender);

  P := 100;

  FillChar (Q, SizeOf(Q), 0);
  Q.IOCODE := RZ_DRIVER_QUERY_PROC_REGISTER;
  Q.PROC   := Int64(@register_beep);
  Q.paramA := Int64(@P);

  if RZDriver_IOCTRL (Q) then begin
    message_('SUCCESS: ' + IntToStr(P));
  end else begin
    message_('ERROR: ' + LastErrorMessage);
  end;
end;

{
  -> RCX = AParam
}
procedure fastcall_beep (AParam: Pointer);
{$IFDEF CPUX64}
asm
        // save argument
        push    rcx

        call    base_beep

        // do something with argument
        pop     rcx
        mov     edx, [rcx]
        add     edx, 1 // inc edx : ??? don`t work in some cases
        mov     [rcx], edx
end;
{$ELSE}
asm
        // save argument
        push    ecx

        call    base_beep

        // do something with argument
        pop     ecx
        mov     edx, [ecx]
        add     edx, 1 // inc edx : ??? don`t work in some cases
        mov     [ecx], edx
end;
{$ENDIF}

{
}
procedure TTestForm.BT_fastcallClick(Sender: TObject);
var
  P : DWORD;
  Q : TRZDriverQuery;
begin
  Refreshh (Sender);

  P := 120;

  FillChar (Q, SizeOf(Q), 0);
  Q.IOCODE := RZ_DRIVER_QUERY_PROC_FASTCALL;
  Q.PROC   := Int64(@fastcall_beep);
  Q.paramA := Int64(@P);

  if RZDriver_IOCTRL (Q) then begin
    message_('SUCCESS: ' + IntToStr(P));
  end else begin
    message_('ERROR: ' + LastErrorMessage);
  end;
end;

{
}
procedure stdcall_beep (AParam: Pointer); stdcall;
{$IFDEF CPUX64}
begin
end;
{$ELSE}
asm
        call    base_beep

        // do something with argument
        mov     eax, [AParam]
        mov     edx, [eax]
        add     edx, 1 // inc edx : ??? don`t work in some cases
        mov     [eax], edx
end;
{$ENDIF}

{
  call user-mode stdcall procedure from ring-0
}
procedure TTestForm.BT_stdcallClick(Sender: TObject);
var
  P : DWORD;
  Q : TRZDriverQuery;
begin
  Refreshh (Sender);

  P := 200;

  FillChar (Q, SizeOf(Q), 0);
  Q.IOCODE := RZ_DRIVER_QUERY_PROC_STDCALL;
  Q.PROC   := Int64(@stdcall_beep);
  Q.paramA := Int64(@P);

  if RZDriver_IOCTRL (Q) then begin
    message_('SUCCESS: ' + IntToStr(P));
  end else begin
    message_('ERROR: ' + LastErrorMessage);
  end;
end;

{
}
procedure cdecl_beep (AParam: Pointer); cdecl;
{$IFDEF CPUX64}
begin
end;
{$ELSE}
asm
        call    base_beep

        // do something with argument
        mov     eax, [AParam]
        mov     edx, [eax]
        add     edx, 1 // inc edx : ??? don`t work in some cases
        mov     [eax], edx
end;
{$ENDIF}

{
  call user-mode cdecl procedure from ring-0
}
procedure TTestForm.BT_cdeclClick(Sender: TObject);
var
  P : DWORD;
  Q : TRZDriverQuery;
begin
  Refreshh (Sender);

  P := 300;

  FillChar (Q, SizeOf(Q), 0);
  Q.IOCODE := RZ_DRIVER_QUERY_PROC_CDECL;
  {$IFNDEF CPUX64}
  Q.PROC   := Int64(@cdecl_beep);
  Q.paramA := Int64(@P);
  {$ENDIF}

  if RZDriver_IOCTRL (Q) then begin
    message_('SUCCESS: ' + IntToStr(P));
  end else begin
    message_('ERROR: ' + LastErrorMessage);
  end;
end;

{
  direct port I/O test
}
procedure TTestForm.BT_PlaySoundClick(Sender: TObject);
var
  VFreq  : WORD;
  VDelay : DWORD;
begin
  Refreshh (Sender);
  
  VFreq := StrToIntDef (ED_Freq.Text, 1500);
  VDelay := StrToIntDef (ED_Delay.Text, 1000);
  PlaySound (VFreq, VDelay);
end;

var
  CCycle : Integer = 0;

{
}
procedure TTestForm.BT_BeepClick(Sender: TObject);
var
  VFreq  : WORD;
begin
  Refreshh (Sender);

  VFreq := StrToIntDef (ED_Freq.Text, 1500);

  case (CCycle mod 3) of
    0 : begin
      IO_RZBeep_start := RZ_P_IO_RZBeep_start;
      IO_RZBeep_stop  := RZ_P_IO_RZBeep_stop;
      IO_RZBeep       := RZ_P_IO_RZBeep;
    end;
    1 : begin
      IO_RZBeep_start := RZ_D_IO_RZBeep_start_REGISTER;
      IO_RZBeep_stop  := RZ_D_IO_RZBeep_stop_REGISTER;
      IO_RZBeep       := nil;
    end;
    2 : begin
      IO_RZBeep_start := RZ_D_IO_RZBeep_start_FASTCALL;
      IO_RZBeep_stop  := RZ_D_IO_RZBeep_stop_FASTCALL;
      IO_RZBeep       := nil;
    end;
  end;
  Inc (CCycle);

  RZBeepStart (VFreq);
  SLEEP (200);
  RZBeepStop;
end;

{
}
procedure TTestForm.BT_BIOSDumpClick(Sender: TObject);
var
  Q      : TRZDriverIOQuery;
  VSRC   : Int64;
  VBYTES : Int64;
  VDST   : Pointer;
  S      : string;
  SFile  : string;
  VF     : THandle;
  VB     : DWORD;
begin
  Refreshh (Sender);

  VSRC := $FFFF0000;
  VBYTES := $10000;

  VDST := GetMemory (VBYTES);
  try
    FillChar (Q, SizeOf(Q), 0);
    Q.IOCODE := RZ_DRIVER_QUERY_MAP_IO_SPACE;
    Q.SRC    := VSRC;
    Q.DST    := DWORD(VDST);
    Q.BYTES  := VBYTES;

    if RZDriver_IOCTRL (Q) then begin
      message_('SUCCESS');

      S := RZDriver.drvname + '_BIOS.BIN';
      SFile := ExtractFilePath (ParamStr(0)) + S;

      VF := CreateFile (PChar(SFile), GENERIC_READ or GENERIC_WRITE,
        FILE_SHARE_READ, nil, CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE, 0);

      if VF <> INVALID_HANDLE_VALUE then try
        WriteFile (VF, PAnsiChar(VDST)^, VBYTES, VB, nil);
        message_('SAVED TO ' + S);
      finally
        CloseHandle (VF);
      end;
    end else begin
      message_('ERROR: ' + LastErrorMessage);
    end;
  finally
    FreeMemory (VDST);
  end;
end;

{
}
procedure TTestForm.BT_ProcessHookSetClick(Sender: TObject);
begin
  Refreshh (Sender);
  ProcessHook_SET;
end;

{
}
procedure TTestForm.BT_ProcessHookRetClick(Sender: TObject);
begin
  Refreshh (Sender);
  ProcessHook_RET;
end;

end.

