unit acDriverUtils;

// written by @L.CHEMIST (Andrey A. Meshkov)
// http://maalchemist.narod.ru
// maalchemist@yandex.ru
// maalchemist@gmail.com

interface

uses
  Windows, WinSvc, acDriverTypes;

function  OpenDriver (var ADriver: TAC_Driver; const ADrvname, ADrvFilename: string): THandle;
procedure CloseDriver (var ADriver: TAC_Driver; AUninstall: Boolean);
function  UninstallDriver (const ADrivername: string): Boolean;

var
  LastError : DWORD = 0;

implementation

{
}
function OpenDriver (var ADriver: TAC_Driver; const ADrvname, ADrvFilename: string): THandle;
var
  L          : Integer;
  P          : PChar;
  S          : string;
  VManager   : SC_HANDLE;
  VDrvPath   : string;
  VDrvDevice : string;
  VVectors   : PChar;
  VStatus    : TServiceStatus;
begin
  LastError := 0;

  Result := ADriver.handle;
  if ADriver.handle <> INVALID_HANDLE_VALUE then begin
    Exit;
  end;

  VManager := OpenSCManager (nil, nil, SC_MANAGER_ALL_ACCESS);
  if VManager = 0 then Exit;

  try
    // find (create) driver service
    ADriver.service := OpenService (VManager, PChar(ADrvname), SERVICE_ALL_ACCESS);
    if ADriver.service = 0 then begin
      VDrvPath := ADrvFilename;
      L := GetFullPathname (PChar(ADrvFilename), 0, PChar(S), P);
      if L > 0 then begin
        SetLength(S, L);
        L := GetFullPathname (PChar(ADrvFilename), Length(S), PChar(S), P);
        if L > 0 then SetString (VDrvPath, PChar(S), L);
      end;

      ADriver.service := CreateService (VManager, PChar(ADrvname), PChar(ADrvname),
        SERVICE_ALL_ACCESS, SERVICE_KERNEL_DRIVER, SERVICE_DEMAND_START, SERVICE_ERROR_NORMAL,
        PChar(VDrvPath), nil, nil, nil, nil, nil);
    end;
    if ADriver.service = 0 then Exit;

    // run driver service
    if not QueryServiceStatus (ADriver.service, VStatus) then Exit;
    if VStatus.dwCurrentState <> SERVICE_RUNNING then begin
      VVectors := nil;
      SetLastError(0);
      if not StartService (ADriver.service, 0, VVectors) then begin
        LastError := GetLastError;
        Exit;
      end;
    end;

    // create driver handle
    VDrvDevice := '\\.\' + ADrvname;
    ADriver.handle := CreateFile (PChar(VDrvDevice), GENERIC_READ or GENERIC_WRITE,
      FILE_SHARE_READ or FILE_SHARE_WRITE, nil, OPEN_EXISTING, 0, 0);
    Result := ADriver.handle;

    if ADriver.handle <> INVALID_HANDLE_VALUE then begin
      ADriver.drvname := ADrvname;
      ADriver.drvpath := VDrvPath;
    end else begin
      LastError := GetLastError;
    end;
  finally
    CloseServiceHandle (VManager);
  end;
end;

{
}
procedure CloseDriver (var ADriver: TAC_Driver; AUninstall: Boolean);
var
  VStatus : TServiceStatus;
begin
  if ADriver.handle <> INVALID_HANDLE_VALUE then begin
    CloseHandle (ADriver.handle);                                    // close driver handle
    ADriver.handle := INVALID_HANDLE_VALUE;
  end;

  if ADriver.service <> 0 then begin
    ControlService (ADriver.service, SERVICE_CONTROL_STOP, VStatus); // stop driver service
    if AUninstall then  DeleteService (ADriver.service);             // mark driver for delete
    CloseServiceHandle (ADriver.service);                            // close (delete) driver
    ADriver.service := 0;
  end;

  if (ADriver.handle = INVALID_HANDLE_VALUE) and (ADriver.service = 0) then begin
    ADriver.drvname := '';
    ADriver.drvpath := '';
  end;
end;

{
  <- true,       
}
function UninstallDriver (const ADrivername: string): Boolean;
var
  VManager : SC_HANDLE;
  VService : SC_HANDLE;
  VStatus  : TServiceStatus;
begin
  Result := false;

  VManager := OpenSCManager (nil, nil, SC_MANAGER_ALL_ACCESS);  // open service manager
  if VManager <> 0 then begin
    VService := OpenService (VManager, PChar(ADrivername), SERVICE_ALL_ACCESS);
    if VService <> 0 then begin                                 // uninstall driver if exists
      ControlService (VService, SERVICE_CONTROL_STOP, VStatus); // stop driver
      DeleteService (VService);                                 // mark driver for delete
      CloseServiceHandle (VService);                            // delete driver
      VService := OpenService (VManager, PChar(ADrivername), SERVICE_ALL_ACCESS);
    end;
    Result := (VService = 0) and (GetLastError = ERROR_SERVICE_DOES_NOT_EXIST);
    CloseServiceHandle (VManager);                              // close service manager
  end;
end;

end.
