unit acRZBeep;

// written by @L.CHEMIST (Andrey A. Meshkov)
// http://maalchemist.narod.ru
// maalchemist@yandex.ru
// maalchemist@gmail.com

interface

uses
  Windows;

type
  TRZBeepProc = procedure;

type
  PRZBeepParams = ^TRZBeepParams;
  TRZBeepParams = record
    Tone   : DWORD;
    Count  : DWORD;
    Delay  : DWORD;
    Result : DWORD;
  end;

procedure RZBeepStart (ATone: Integer);
procedure RZBeepStop;
procedure RZBeep (ATone, ACount, ADelay: Integer);

procedure D_IO_RZBeep_start;
procedure D_IO_RZBeep_start_REGISTER;
procedure D_IO_RZBeep_start_FASTCALL;
procedure RZ_P_IO_RZBeep_start;
procedure RZ_D_IO_RZBeep_start_REGISTER;
procedure RZ_D_IO_RZBeep_start_FASTCALL;

procedure D_IO_RZBeep_stop;
procedure D_IO_RZBeep_stop_REGISTER;
procedure D_IO_RZBeep_stop_FASTCALL;
procedure RZ_P_IO_RZBeep_stop;
procedure RZ_D_IO_RZBeep_stop_REGISTER;
procedure RZ_D_IO_RZBeep_stop_FASTCALL;

procedure D_IO_RZBeep;
procedure D_IO_RZBeep_REGISTER;
procedure D_IO_RZBeep_FASTCALL;
procedure RZ_P_IO_RZBeep;
procedure RZ_D_IO_RZBeep_REGISTER;
procedure RZ_D_IO_RZBeep_FASTCALL;

var
  IO_RZBeep_start : TRZBeepProc = nil;
  IO_RZBeep_stop  : TRZBeepProc = nil;
  IO_RZBeep       : TRZBeepProc = nil;

implementation

uses
  acRZDriver, acRZDriverTypes, acRZIO;

const
  MAXBeepTone  = 65535;
  DEFBeepTone  = 600;
  DEFBeepCount = 30000;
  DEFBeepDelay = 3000;

var
  FBeepTone  : WORD  = DEFBeepTone;
  FBeepCount : DWORD = DEFBeepCount;
  FBeepDelay : DWORD = DEFBeepDelay;

{
}
procedure RZBeepStart (ATone: Integer);
var
  VTone : Integer;
begin
  if not Assigned (IO_RZBeep_start) then Exit;

  try
    VTone := ATone;
    if VTone <= 0 then VTone := DEFBeepTone;
    if VTone > MAXBeepTone then VTone := MAXBeepTone;
    FBeepTone := VTone;

    FBeepCount := 0;
    FBeepDelay := 0;

    IO_RZBeep_start;
  except
    // raise;
  end;
end;

{
}
procedure RZBeepStop;
begin
  if not Assigned (IO_RZBeep_stop) then Exit;

  try
    IO_RZBeep_stop;
  except
    // raise;
  end;
end;

{
}
procedure RZBeep (ATone, ACount, ADelay: Integer);
var
  VTone  : Integer;
  VCount : Integer;
  VDelay : Integer;
begin
  if not Assigned (IO_RZBeep) then Exit;

  try
    VTone := ATone;
    if VTone <= 0 then VTone := DEFBeepTone;
    if VTone > MAXBeepTone then VTone := MAXBeepTone;
    FBeepTone := VTone;

    VCount := ACount;
    if VCount < 0 then VCount := DEFBeepCount;
    FBeepCount := VCount;

    VDelay := ADelay;
    if VDelay < 0 then VDelay := DEFBeepDelay;
    FBeepDelay := VDelay;

    IO_RZBeep;
  except
    // raise;
  end;
end;

{
  direct I/O
}
procedure D_IO_RZBeep_start;
asm
        mov     al, 0B6h
        out     43h, al
        in      al, 61h
        or      al, 3
        out     61h, al

        mov     ax, FBeepTone
        out     42h, al
        mov     al, ah
        out     42h, al
end;

{
  direct I/O
  [IN] - EAX = PBeepParams
}
procedure D_IO_RZBeep_start_REGISTER;
asm
        {$IFDEF CPUX64}
        push    rsi
        mov     rsi, rax // RSI = PBeepParams
        {$ELSE}
        push    esi
        mov     esi, eax // ESI = PBeepParams
        {$ENDIF}

        mov     al, 0B6h
        out     43h, al
        in      al, 61h
        or      al, 3
        out     61h, al

        mov     ax, WORD PTR [esi + TRZBeepParams.Tone]
        out     42h, al
        mov     al, ah
        out     42h, al

    @RESULT:
        mov     DWORD PTR [esi + TRZBeepParams.Result], 1

        {$IFDEF CPUX64}
        pop     rsi
        {$ELSE}
        pop     esi
        {$ENDIF}
end;

{
  direct I/O
  [IN] - ECX = PBeepParams
}
procedure D_IO_RZBeep_start_FASTCALL;
asm
        {$IFDEF CPUX64}
        push    rsi
        mov     rsi, rcx // RSI = PBeepParams
        {$ELSE}
        push    esi
        mov     esi, ecx // ESI = PBeepParams
        {$ENDIF}

        mov     al, 0B6h
        out     43h, al
        in      al, 61h
        or      al, 3
        out     61h, al

        mov     ax, WORD PTR [esi + TRZBeepParams.Tone]
        out     42h, al
        mov     al, ah
        out     42h, al

    @RESULT:
        mov     DWORD PTR [esi + TRZBeepParams.Result], 1

        {$IFDEF CPUX64}
        pop     rsi
        {$ELSE}
        pop     esi
        {$ENDIF}
end;

{
  RZ port I/O
}
procedure RZ_P_IO_RZBeep_start;
asm
        mov     al, 0B6h

        (* maa
        out     43h, al // *)
        mov     dx, 43h
        call    RZ_OUT_BYTE_SAFE

        (* maa
        in      al, 61h // *)
        mov     dx, 61h
        call    RZ_IN_BYTE_SAFE

        or      al, 3

        (* maa
        out     61h, al // *)
        mov     dx, 61h
        call    RZ_OUT_BYTE_SAFE

        mov     ax, FBeepTone
        (* maa
        out     42h, al // *)
        mov     dx, 42h
        call    RZ_OUT_BYTE_SAFE

        mov     al, ah

        (* maa
        out     42h, al // *)
        mov     dx, 42h
        call    RZ_OUT_BYTE_SAFE
end;

{
  direct I/O
}
procedure D_IO_RZBeep_stop;
asm
        and     al, 11111100b
        out     61h, al
end;

{
  direct I/O
}
procedure D_IO_RZBeep_stop_REGISTER;
asm
        and     al, 11111100b
        out     61h, al
end;

{
  direct I/O
}
procedure D_IO_RZBeep_stop_FASTCALL;
asm
        and     al, 11111100b
        out     61h, al
end;

{
  RZ port I/O
}
procedure RZ_P_IO_RZBeep_stop;
asm
        and     al, 11111100b

        (* maa
        out     61h, al // *)
        mov     dx, 61h
        call    RZ_OUT_BYTE_SAFE
end;

{
  direct I/O
}
procedure D_IO_RZBeep;
asm
        mov     al, 0B6h
        out     43h, al
        in      al, 61h
        or      al, 3
        out     61h, al

        mov     ax, FBeepTone
        mov     ecx, FBeepCount
        test    ecx, ecx
        jz      @MUSICOUT
    @MUSIC:
        {$IFDEF CPUX64}
        push    rcx
        {$ELSE}
        push    ecx
        {$ENDIF}

        out     42h, al
        mov     al, ah
        out     42h, al

        mov     ecx, FBeepDelay
        test    ecx, ecx
        jz      @DELAYOUT
    @DELAY:
        sub     ecx, 1 // dec ecx : ??? don`t work in some cases
        jnz     @DELAY
    @DELAYOUT:
        {$IFDEF CPUX64}
        pop     rcx
        {$ELSE}
        pop     ecx
        {$ENDIF}
        sub     ecx, 1 // dec ecx : ??? don`t work in some cases
        jnz     @MUSIC

    @MUSICOUT:
        and     al, 11111100b
        out     61h, al
end;

{
  direct I/O
  [IN] - EAX = PBeepParams
}
procedure D_IO_RZBeep_REGISTER;
asm
        {$IFDEF CPUX64}
        push    rsi
        mov     rsi, rax // RSI = PBeepParams
        {$ELSE}
        push    esi
        mov     esi, eax // ESI = PBeepParams
        {$ENDIF}

        mov     al, 0B6h
        out     43h, al
        in      al, 61h
        or      al, 3
        out     61h, al

        mov     ax, WORD PTR [esi + TRZBeepParams.Tone]
        mov     ecx, DWORD PTR [esi + TRZBeepParams.Count]
        test    ecx, ecx
        jz      @MUSICOUT
    @MUSIC:
        {$IFDEF CPUX64}
        push    rcx
        {$ELSE}
        push    ecx
        {$ENDIF}

        out     42h, al
        mov     al, ah
        out     42h, al

        mov     ecx, DWORD PTR [esi + TRZBeepParams.Delay]
        test    ecx, ecx
        jz      @DELAYOUT
    @DELAY:
        sub     ecx, 1 // dec ecx : ??? don`t work in some cases
        jnz     @DELAY
    @DELAYOUT:
        {$IFDEF CPUX64}
        pop     rcx
        {$ELSE}
        pop     ecx
        {$ENDIF}
        sub     ecx, 1 // dec ecx : ??? don`t work in some cases
        jnz     @MUSIC

    @MUSICOUT:
        and     al, 11111100b
        out     61h, al

    @RESULT:
        mov     DWORD PTR [esi + TRZBeepParams.Result], 1

        {$IFDEF CPUX64}
        pop     rsi
        {$ELSE}
        pop     esi
        {$ENDIF}
end;

{
  direct I/O
  [IN] - ECX = PBeepParams
}
procedure D_IO_RZBeep_FASTCALL;
asm
        {$IFDEF CPUX64}
        push    rsi
        mov     rsi, rcx // RSI = PBeepParams
        {$ELSE}
        push    esi
        mov     esi, ecx // ESI = PBeepParams
        {$ENDIF}

        mov     al, 0B6h
        out     43h, al
        in      al, 61h
        or      al, 3
        out     61h, al

        mov     ax, WORD PTR [esi + TRZBeepParams.Tone]
        mov     ecx, DWORD PTR [esi + TRZBeepParams.Count]
        test    ecx, ecx
        jz      @MUSICOUT
    @MUSIC:
        {$IFDEF CPUX64}
        push    rcx
        {$ELSE}
        push    ecx
        {$ENDIF}

        out     42h, al
        mov     al, ah
        out     42h, al

        mov     ecx, DWORD PTR [esi + TRZBeepParams.Delay]
        test    ecx, ecx
        jz      @DELAYOUT
    @DELAY:
        sub     ecx, 1 // dec ecx : ??? don`t work in some cases
        jnz     @DELAY
    @DELAYOUT:
        {$IFDEF CPUX64}
        pop     rcx
        {$ELSE}
        pop     ecx
        {$ENDIF}
        sub     ecx, 1 // dec ecx : ??? don`t work in some cases
        jnz     @MUSIC

    @MUSICOUT:
        and     al, 11111100b
        out     61h, al

    @RESULT:
        mov     DWORD PTR [esi + TRZBeepParams.Result], 1

        {$IFDEF CPUX64}
        pop     rsi
        {$ELSE}
        pop     esi
        {$ENDIF}
end;

{
  RZ port I/O
}
procedure RZ_P_IO_RZBeep;
asm
        mov     al, 0B6h

        (* maa
        out     43h, al // *)
        mov     dx, 43h
        call    RZ_OUT_BYTE_SAFE

        (* maa
        in      al, 61h // *)
        mov     dx, 61h
        call    RZ_IN_BYTE_SAFE

        or      al, 3

        (* maa
        out     61h, al // *)
        mov     dx, 61h
        call    RZ_OUT_BYTE_SAFE

        mov     ax, FBeepTone
        mov     ecx, FBeepCount
        test    ecx, ecx
        jz      @MUSICOUT
    @MUSIC:
        {$IFDEF CPUX64}
        push    rcx
        {$ELSE}
        push    ecx
        {$ENDIF}

        (* maa
        out     42h, al // *)
        mov     dx, 42h
        call    RZ_OUT_BYTE_SAFE

        mov     al, ah

        (* maa
        out     42h, al // *)
        mov     dx, 42h
        call    RZ_OUT_BYTE_SAFE

        mov     ecx, FBeepDelay
        test    ecx, ecx
        jz      @DELAYOUT
    @DELAY:
        sub     ecx, 1 // dec ecx : ??? don`t work in some cases
        jnz     @DELAY
    @DELAYOUT:
        {$IFDEF CPUX64}
        pop     rcx
        {$ELSE}
        pop     ecx
        {$ENDIF}
        sub     ecx, 1 // dec ecx : ??? don`t work in some cases
        jnz     @MUSIC

    @MUSICOUT:
        and     al, 11111100b

        (* maa
        out     61h, al // *)
        mov     dx, 61h
        call    RZ_OUT_BYTE_SAFE
end;

{
  RZ direct I/O
}
procedure RZ_D_IO_RZBeep_start_REGISTER;
var
  Q : TRZDriverQuery;
  P : TRZBeepParams;
begin
  P.Tone   := FBeepTone;
  P.Count  := FBeepCount;
  P.Delay  := FBeepDelay;
  P.Result := 0;

  Q.IOCODE := RZ_DRIVER_QUERY_PROC_REGISTER;
  Q.PROC   := Int64(@D_IO_RZBeep_start_REGISTER);
  Q.paramA := Int64(@P);

  RZDriver_IOCTRL (Q);
end;

{
  RZ direct I/O
}
procedure RZ_D_IO_RZBeep_start_FASTCALL;
var
  Q : TRZDriverQuery;
  P : TRZBeepParams;
begin
  P.Tone   := FBeepTone;
  P.Count  := FBeepCount;
  P.Delay  := FBeepDelay;
  P.Result := 0;

  Q.IOCODE := RZ_DRIVER_QUERY_PROC_FASTCALL;
  Q.PROC   := Int64(@D_IO_RZBeep_start_FASTCALL);
  Q.paramA := Int64(@P);

  RZDriver_IOCTRL (Q);
end;

{
  RZ direct I/O
}
procedure RZ_D_IO_RZBeep_stop_REGISTER;
var
  Q : TRZDriverQuery;
  P : TRZBeepParams;
begin
  P.Tone   := 0;
  P.Count  := 0;
  P.Delay  := 0;
  P.Result := 0;

  Q.IOCODE := RZ_DRIVER_QUERY_PROC_REGISTER;
  Q.PROC   := Int64(@D_IO_RZBeep_stop_REGISTER);
  Q.paramA := Int64(@P);

  RZDriver_IOCTRL (Q);
end;

{
  RZ direct I/O
}
procedure RZ_D_IO_RZBeep_stop_FASTCALL;
var
  Q : TRZDriverQuery;
  P : TRZBeepParams;
begin
  P.Tone   := 0;
  P.Count  := 0;
  P.Delay  := 0;
  P.Result := 0;

  Q.IOCODE := RZ_DRIVER_QUERY_PROC_FASTCALL;
  Q.PROC   := Int64(@D_IO_RZBeep_stop_FASTCALL);
  Q.paramA := Int64(@P);

  RZDriver_IOCTRL (Q);
end;

{
  RZ direct I/O
}
procedure RZ_D_IO_RZBeep_REGISTER;
var
  Q : TRZDriverQuery;
  P : TRZBeepParams;
begin
  P.Tone   := FBeepTone;
  P.Count  := FBeepCount;
  P.Delay  := FBeepDelay;
  P.Result := 0;

  Q.IOCODE := RZ_DRIVER_QUERY_PROC_REGISTER;
  Q.PROC   := Int64(@D_IO_RZBeep_REGISTER);
  Q.paramA := Int64(@P);

  RZDriver_IOCTRL (Q);
end;

{
  RZ direct I/O
}
procedure RZ_D_IO_RZBeep_FASTCALL;
var
  Q : TRZDriverQuery;
  P : TRZBeepParams;
begin
  P.Tone   := FBeepTone;
  P.Count  := FBeepCount;
  P.Delay  := FBeepDelay;
  P.Result := 0;

  Q.IOCODE := RZ_DRIVER_QUERY_PROC_FASTCALL;
  Q.PROC   := Int64(@D_IO_RZBeep_FASTCALL);
  Q.paramA := Int64(@P);

  RZDriver_IOCTRL (Q);
end;

end.

