unit acRZDriverTypes;

// written by @L.CHEMIST (Andrey A. Meshkov)
// http://maalchemist.narod.ru
// maalchemist@yandex.ru
// maalchemist@gmail.com

interface

uses
  Windows, acDriverTypes;

const
  RZ_DRIVER_BUFFER_CAPACITY               = 64;
  RZ_DRIVER_QUERY_BUFFER_SIZE             = 24;

  RZ_DRIVER_QUERY_EMPTY                   = $0000; //
  RZ_DRIVER_QUERY_NULL                    = $0000; //

  RZ_DRIVER_QUERY_PROC_NOARGS             = $0010; // ...
  RZ_DRIVER_QUERY_PROC_REGISTER           = $0011; // paramA->eax(rax) ...
  RZ_DRIVER_QUERY_PROC_FASTCALL           = $0012; // paramA->ecx(rcx) ...
  RZ_DRIVER_QUERY_PROC_STDCALL            = $0013; // push paramA ...
  RZ_DRIVER_QUERY_PROC_CDECL              = $0014; // push paramA ...

  RZ_DRIVER_QUERY_PROC_NOARGS_IFRAME      = $0110; // cli ... sti
  RZ_DRIVER_QUERY_PROC_REGISTER_IFRAME    = $0111; // cli paramA->eax(rax) ... sti
  RZ_DRIVER_QUERY_PROC_FASTCALL_IFRAME    = $0112; // cli paramA->ecx(rcx) ... sti
  RZ_DRIVER_QUERY_PROC_STDCALL_IFRAME     = $0113; // cli push paramA ... sti
  RZ_DRIVER_QUERY_PROC_CDECL_IFRAME       = $0114; // cli push paramA ... sti

  RZ_DRIVER_QUERY_PROC_REGISTER_2P        = $0015; // paramA->eax(rax), paramB->edx(rdx) ...
  RZ_DRIVER_QUERY_PROC_FASTCALL_2P        = $0016; // paramA->ecx(rcx), paramB->edx(rdx) ...
  RZ_DRIVER_QUERY_PROC_STDCALL_2P         = $0017; // push paramB, push paramA ...
  RZ_DRIVER_QUERY_PROC_CDECL_2P           = $0018; // push paramB, push paramA ...

  RZ_DRIVER_QUERY_PROC_REGISTER_2P_IFRAME = $0115; // cli paramA->eax(rax), paramB->edx(rdx) ... sti
  RZ_DRIVER_QUERY_PROC_FASTCALL_2P_IFRAME = $0116; // cli paramA->ecx(rcx), paramB->edx(rdx) ... sti
  RZ_DRIVER_QUERY_PROC_STDCALL_2P_IFRAME  = $0117; // cli push paramB, push paramA ... sti
  RZ_DRIVER_QUERY_PROC_CDECL_2P_IFRAME    = $0118; // cli push paramB, push paramA ... sti

  RZ_DRIVER_QUERY_PORT_IN_BYTE            = $0020; //
  RZ_DRIVER_QUERY_PORT_IN_WORD            = $0021; //
  RZ_DRIVER_QUERY_PORT_IN_DWORD           = $0022; //

  RZ_DRIVER_QUERY_PORT_OUT_BYTE           = $0030; //
  RZ_DRIVER_QUERY_PORT_OUT_WORD           = $0031; //
  RZ_DRIVER_QUERY_PORT_OUT_DWORD          = $0033; //

  RZ_DRIVER_QUERY_MAP_IO_SPACE            = $0040; // ...
  RZ_DRIVER_QUERY_MAP_IO_SPACE_IFRAME     = $0041; // cli ... sti
  RZ_DRIVER_QUERY_UNMAP_IO_SPACE          = $0042; // ...
  RZ_DRIVER_QUERY_UNMAP_IO_SPACE_IFRAME   = $0043; // cli ... sti

  RZ_DRIVER_QUERY_CP_SET                  = $0050; // ...
  RZ_DRIVER_QUERY_CP_SET_IFRAME           = $0051; // cli ... sti
  RZ_DRIVER_QUERY_CP_RET                  = $0052; // ...
  RZ_DRIVER_QUERY_CP_RET_IFRAME           = $0053; // cli ... sti
  RZ_DRIVER_QUERY_CP_GET                  = $0054; // ...
  RZ_DRIVER_QUERY_CP_GET_IFRAME           = $0055; // cli ... sti

  RZ_DRIVER_QUERY_LI_SET                  = $0060; // ...
  RZ_DRIVER_QUERY_LI_SET_IFRAME           = $0061; // cli ... sti
  RZ_DRIVER_QUERY_LI_RET                  = $0062; // ...
  RZ_DRIVER_QUERY_LI_RET_IFRAME           = $0063; // cli ... sti
  RZ_DRIVER_QUERY_LI_GET                  = $0064; // ...
  RZ_DRIVER_QUERY_LI_GET_IFRAME           = $0065; // cli ... sti

type
  PRZDriverQueryBuffer = ^TRZDriverQueryBuffer;
  TRZDriverQueryBuffer = array [0..RZ_DRIVER_QUERY_BUFFER_SIZE-1] of BYTE;

  // base
  PRZDriverQuery = ^TRZDriverQuery;
  TRZDriverQuery = packed record
    IOCODE : DWORD; // user I/O code
    IODATA : DWORD; // user I/O data
    PROC   : Int64; // procedure offset
    paramA : Int64; // parameter A
    paramB : Int64; // parameter B
    RESULT : Int64; // result
    Buffer : TRZDriverQueryBuffer;
  end;

  // I/O, memory map
  PRZDriverIOQuery = ^TRZDriverIOQuery;
  TRZDriverIOQuery = packed record
    IOCODE : DWORD; // user I/O code
    IODATA : DWORD; // user I/O data
    SRC    : Int64; // source address
    DST    : Int64; // destination address
    BYTES  : Int64; // memory size in bytes
    RESULT : Int64; // result
    Buffer : TRZDriverQueryBuffer;
  end;

  // create process
  PRZDriverCPQuery = ^TRZDriverCPQuery;
  TRZDriverCPQuery = packed record
    IOCODE    : DWORD; // user I/O code
    IODATA    : DWORD; // user I/O data
    ParentID  : Int64; // parent process ID
    ProcessID : Int64; // process ID
    Create    : Int64; // create state
    RESULT    : Int64; // result
    Buffer    : TRZDriverQueryBuffer;
  end;
  PRZDriverCPQueries = ^TRZDriverCPQueries;
  TRZDriverCPQueries = array [0..RZ_DRIVER_BUFFER_CAPACITY - 1] of TRZDriverCPQuery;

  PRZDriverCPQueryResult = ^TRZDriverCPQueryResult;
  TRZDriverCPQueryResult = packed record
    CP_RecvCount : DWORD;
    CP_BuffDepth : WORD;
    CP_DropCount : WORD;
  end;

  // load image
  PRZDriverLIQuery = ^TRZDriverLIQuery;
  TRZDriverLIQuery = packed record
    IOCODE    : DWORD; // user I/O code
    IODATA    : DWORD; // user I/O data
    ProcessID : Int64; // process ID
    Image     : Int64; // image name buffer address
    Reserved  : Int64; // reserved
    RESULT    : Int64; // result
    Buffer    : TRZDriverQueryBuffer;
  end;

implementation

end.
