unit acRZIO;

// written by @L.CHEMIST (Andrey A. Meshkov)
// http://maalchemist.narod.ru
// maalchemist@yandex.ru
// maalchemist@gmail.com

interface

uses
  Windows;

function  RZ_IN_BYTE: BYTE;
function  RZ_IN_BYTE_SAFE: BYTE;

function  RZ_IN_WORD: WORD;
function  RZ_IN_WORD_SAFE: WORD;

function  RZ_IN_DWORD: DWORD;
function  RZ_IN_DWORD_SAFE: DWORD;

procedure RZ_OUT_BYTE;
procedure RZ_OUT_BYTE_SAFE;

procedure RZ_OUT_WORD;
procedure RZ_OUT_WORD_SAFE;

procedure RZ_OUT_DWORD;
procedure RZ_OUT_DWORD_SAFE;

function  PortInB (APort: WORD): BYTE;
function  PortInW (APort: WORD): WORD;
function  PortInD (APort: WORD): DWORD;

procedure PortOutB (APort: WORD; AValue: BYTE);
procedure PortOutW (APort: WORD; AValue: WORD);
procedure PortOutD (APort: WORD; AValue: DWORD);

procedure SpeakerOn (ATone: WORD);
procedure SpeakerOff;
procedure PlaySound (AFrequency: WORD; ADelay: DWORD);

implementation

uses
  acRZDriver, acRZDriverTypes;

var
  PortInBQuery : TRZDriverQuery = (
    IOCODE : RZ_DRIVER_QUERY_PORT_IN_BYTE;
    IODATA : 0; PROC : 0; paramA : 0; paramB : 0; RESULT : 0;
  );

  PortInWQuery : TRZDriverQuery = (
    IOCODE : RZ_DRIVER_QUERY_PORT_IN_WORD;
    IODATA : 0; PROC : 0; paramA : 0; paramB : 0; RESULT : 0;
  );

  PortInDQuery : TRZDriverQuery = (
    IOCODE : RZ_DRIVER_QUERY_PORT_IN_DWORD;
    IODATA : 0; PROC : 0; paramA : 0; paramB : 0; RESULT : 0;
  );

  PortOutBQuery : TRZDriverQuery = (
    IOCODE : RZ_DRIVER_QUERY_PORT_OUT_BYTE;
    IODATA : 0; PROC : 0; paramA : 0; paramB : 0; RESULT : 0;
  );

  PortOutWQuery : TRZDriverQuery = (
    IOCODE : RZ_DRIVER_QUERY_PORT_OUT_WORD;
    IODATA : 0; PROC : 0; paramA : 0; paramB : 0; RESULT : 0;
  );

  PortOutDQuery : TRZDriverQuery = (
    IOCODE : RZ_DRIVER_QUERY_PORT_OUT_DWORD;
    IODATA : 0; PROC : 0; paramA : 0; paramB : 0; RESULT : 0;
  );

{
}
function RZ_IN_BYTE: BYTE;
//  read byte from port (in al, dx)
// ----------------------------
//  [in ] dx - port
//  [out] al - port byte value
// ----------------------------
var
  Q : TRZDriverQuery;
asm
    MOV  DWORD PTR [Q.IOCODE], RZ_DRIVER_QUERY_PORT_IN_BYTE
    MOV  WORD PTR [Q.paramA], DX
    {$IFDEF CPUX64}
    LEA  RCX, [Q]
    {$ELSE}
    LEA  EAX, [Q]
    {$ENDIF}
    CALL RZDriver_IOCTRL
    TEST EAX, EAX
    JZ   @R
    MOV  AL, BYTE PTR [Q.RESULT]
    @R:
end;

{
}
function RZ_IN_BYTE_SAFE: BYTE;
//  read byte from port (in al, dx)
// ----------------------------
//  [in ] dx - port
//  [out] al - port byte value
// ----------------------------
var
  Q : TRZDriverQuery;
asm
    {$IFDEF CPUX64}
    PUSH RCX
    PUSH RDX
    PUSH RBX
    {$ELSE}
    PUSH ECX
    PUSH EDX
    PUSH EBX
    {$ENDIF}

    MOV  DWORD PTR [Q.IOCODE], RZ_DRIVER_QUERY_PORT_IN_BYTE
    MOV  WORD PTR [Q.paramA], DX
    {$IFDEF CPUX64}
    LEA  RCX, [Q]
    {$ELSE}
    LEA  EAX, [Q]
    {$ENDIF}
    CALL RZDriver_IOCTRL
    TEST EAX, EAX
    JZ   @R
    MOV  AL, BYTE PTR [Q.RESULT]
    @R:

    {$IFDEF CPUX64}
    POP  RBX
    POP  RDX
    POP  RCX
    {$ELSE}
    POP  EBX
    POP  EDX
    POP  ECX
    {$ENDIF}
end;

{
}
function RZ_IN_WORD: WORD;
//  read word from port (in ax, dx)
// ----------------------------
//  [in ] dx - port
//  [out] al - port word value
// ----------------------------
var
  Q : TRZDriverQuery;
asm
    MOV  DWORD PTR [Q.IOCODE], RZ_DRIVER_QUERY_PORT_IN_WORD
    MOV  WORD PTR [Q.paramA], DX
    {$IFDEF CPUX64}
    LEA  RCX, [Q]
    {$ELSE}
    LEA  EAX, [Q]
    {$ENDIF}
    CALL RZDriver_IOCTRL
    TEST EAX, EAX
    JZ   @R
    MOV  AX, WORD PTR [Q.RESULT]
    @R:
end;

{
}
function RZ_IN_WORD_SAFE: WORD;
//  read word from port (in ax, dx)
// ----------------------------
//  [in ] dx - port
//  [out] ax - port word value
// ----------------------------
var
  Q : TRZDriverQuery;
asm
    {$IFDEF CPUX64}
    PUSH RCX
    PUSH RDX
    PUSH RBX
    {$ELSE}
    PUSH ECX
    PUSH EDX
    PUSH EBX
    {$ENDIF}

    MOV  DWORD PTR [Q.IOCODE], RZ_DRIVER_QUERY_PORT_IN_WORD
    MOV  WORD PTR [Q.paramA], DX
    {$IFDEF CPUX64}
    LEA  RCX, [Q]
    {$ELSE}
    LEA  EAX, [Q]
    {$ENDIF}
    CALL RZDriver_IOCTRL
    TEST EAX, EAX
    JZ   @R
    MOV  AX, WORD PTR [Q.RESULT]
    @R:

    {$IFDEF CPUX64}
    POP  RBX
    POP  RDX
    POP  RCX
    {$ELSE}
    POP  EBX
    POP  EDX
    POP  ECX
    {$ENDIF}
end;

{
}
function RZ_IN_DWORD: DWORD;
//  read dword from port (in eax, dx)
// ------------------------------
//  [in ]  dx - port
//  [out] eax - port dword value
// ------------------------------
var
  Q : TRZDriverQuery;
asm
    MOV  DWORD PTR [Q.IOCODE], RZ_DRIVER_QUERY_PORT_IN_DWORD
    MOV  WORD PTR [Q.paramA], DX
    {$IFDEF CPUX64}
    LEA  RCX, [Q]
    {$ELSE}
    LEA  EAX, [Q]
    {$ENDIF}
    CALL RZDriver_IOCTRL
    TEST EAX, EAX
    JZ   @R
    MOV  EAX, DWORD PTR [Q.RESULT]
    @R:
end;

{
}
function RZ_IN_DWORD_SAFE: DWORD;
//  read dword from port (in eax, dx)
// ------------------------------
//  [in ]  dx - port
//  [out] eax - port dword value
// ------------------------------
var
  Q : TRZDriverQuery;
asm
    {$IFDEF CPUX64}
    PUSH RCX
    PUSH RDX
    PUSH RBX
    {$ELSE}
    PUSH ECX
    PUSH EDX
    PUSH EBX
    {$ENDIF}

    MOV  DWORD PTR [Q.IOCODE], RZ_DRIVER_QUERY_PORT_IN_DWORD
    MOV  WORD PTR [Q.paramA], DX
    {$IFDEF CPUX64}
    LEA  RCX, [Q]
    {$ELSE}
    LEA  EAX, [Q]
    {$ENDIF}
    CALL RZDriver_IOCTRL
    TEST EAX, EAX
    JZ   @R
    MOV  EAX, DWORD PTR [Q.RESULT]
    @R:

    {$IFDEF CPUX64}
    POP  RBX
    POP  RDX
    POP  RCX
    {$ELSE}
    POP  EBX
    POP  EDX
    POP  ECX
    {$ENDIF}
end;

{
}
procedure RZ_OUT_BYTE;
//  write byte to port (out dx, al)
// ---------------------------
//  [in] dx - port
//  [in] al - port byte value
// ---------------------------
var
  Q : TRZDriverQuery;
asm
    MOV  DWORD PTR [Q.IOCODE], RZ_DRIVER_QUERY_PORT_OUT_BYTE
    MOV  WORD PTR [Q.paramA], DX
    MOV  BYTE PTR [Q.paramB], AL
    {$IFDEF CPUX64}
    LEA  RCX, [Q]
    {$ELSE}
    LEA  EAX, [Q]
    {$ENDIF}
    CALL RZDriver_IOCTRL
end;

{
}
procedure RZ_OUT_BYTE_SAFE;
//  write byte to port (out dx, al)
// ---------------------------
//  [in] dx - port
//  [in] al - port byte value
// ---------------------------
var
  Q : TRZDriverQuery;
asm
    {$IFDEF CPUX64}
    PUSH RAX
    PUSH RCX
    PUSH RDX
    PUSH RBX
    {$ELSE}
    PUSH EAX
    PUSH ECX
    PUSH EDX
    PUSH EBX
    {$ENDIF}

    MOV  DWORD PTR [Q.IOCODE], RZ_DRIVER_QUERY_PORT_OUT_BYTE
    MOV  WORD PTR [Q.paramA], DX
    MOV  BYTE PTR [Q.paramB], AL
    {$IFDEF CPUX64}
    LEA  RCX, [Q]
    {$ELSE}
    LEA  EAX, [Q]
    {$ENDIF}
    CALL RZDriver_IOCTRL

    {$IFDEF CPUX64}
    POP  RBX
    POP  RDX
    POP  RCX
    POP  RAX
    {$ELSE}
    POP  EBX
    POP  EDX
    POP  ECX
    POP  EAX
    {$ENDIF}
end;

{
}
procedure RZ_OUT_WORD;
//  write word to port (out dx, ax)
// ---------------------------
//  [in] dx - port
//  [in] ax - port word value
// ---------------------------
var
  Q : TRZDriverQuery;
asm
    MOV  DWORD PTR [Q.IOCODE], RZ_DRIVER_QUERY_PORT_OUT_WORD
    MOV  WORD PTR [Q.paramA], DX
    MOV  WORD PTR [Q.paramB], AX
    {$IFDEF CPUX64}
    LEA  RCX, [Q]
    {$ELSE}
    LEA  EAX, [Q]
    {$ENDIF}
    CALL RZDriver_IOCTRL
end;

{
}
procedure RZ_OUT_WORD_SAFE;
//  write word to port (out dx, ax)
// ---------------------------
//  [in] dx - port
//  [in] ax - port word value
// ---------------------------
var
  Q : TRZDriverQuery;
asm
    {$IFDEF CPUX64}
    PUSH RAX
    PUSH RCX
    PUSH RDX
    PUSH RBX
    {$ELSE}
    PUSH EAX
    PUSH ECX
    PUSH EDX
    PUSH EBX
    {$ENDIF}

    MOV  DWORD PTR [Q.IOCODE], RZ_DRIVER_QUERY_PORT_OUT_WORD
    MOV  WORD PTR [Q.paramA], DX
    MOV  WORD PTR [Q.paramB], AX
    {$IFDEF CPUX64}
    LEA  RCX, [Q]
    {$ELSE}
    LEA  EAX, [Q]
    {$ENDIF}
    CALL RZDriver_IOCTRL

    {$IFDEF CPUX64}
    POP  RBX
    POP  RDX
    POP  RCX
    POP  RAX
    {$ELSE}
    POP  EBX
    POP  EDX
    POP  ECX
    POP  EAX
    {$ENDIF}
end;

{
}
procedure RZ_OUT_DWORD;
//  write dword to port (out dx, eax)
// -----------------------------
//  [in]  dx - port
//  [in] eax - port dword value
// -----------------------------
var
  Q : TRZDriverQuery;
asm
    MOV  DWORD PTR [Q.IOCODE], RZ_DRIVER_QUERY_PORT_OUT_DWORD
    MOV  WORD PTR [Q.paramA], DX
    MOV  DWORD PTR [Q.paramB], EAX
    {$IFDEF CPUX64}
    LEA  RCX, [Q]
    {$ELSE}
    LEA  EAX, [Q]
    {$ENDIF}
    CALL RZDriver_IOCTRL
end;

{
}
procedure RZ_OUT_DWORD_SAFE;
//  write dword to port (out dx, eax)
// -----------------------------
//  [in]  dx - port
//  [in] eax - port dword value
// -----------------------------
var
  Q : TRZDriverQuery;
asm
    {$IFDEF CPUX64}
    PUSH RAX
    PUSH RCX
    PUSH RDX
    PUSH RBX
    {$ELSE}
    PUSH EAX
    PUSH ECX
    PUSH EDX
    PUSH EBX
    {$ENDIF}

    MOV  DWORD PTR [Q.IOCODE], RZ_DRIVER_QUERY_PORT_OUT_DWORD
    MOV  WORD PTR [Q.paramA], DX
    MOV  DWORD PTR [Q.paramB], EAX
    {$IFDEF CPUX64}
    LEA  RCX, [Q]
    {$ELSE}
    LEA  EAX, [Q]
    {$ENDIF}
    CALL RZDriver_IOCTRL

    {$IFDEF CPUX64}
    POP  RBX
    POP  RDX
    POP  RCX
    POP  RAX
    {$ELSE}
    POP  EBX
    POP  EDX
    POP  ECX
    POP  EAX
    {$ENDIF}
end;

{
  read byte from port
  in al, dx
}
function PortInB (APort: WORD): BYTE;
begin
  Result := 0;
  PortInBQuery.paramA := APort;
  if RZDriver_IOCTRL (PortInBQuery) then Result := PortInBQuery.RESULT;
end;

{
  read word from port
  in ax, dx
}
function PortInW (APort: WORD): WORD;
begin
  Result := 0;
  PortInWQuery.paramA := APort;
  if RZDriver_IOCTRL (PortInWQuery) then Result := PortInWQuery.RESULT;
end;

{
  read dword from port
  in eax, dx
}
function PortInD (APort: WORD): DWORD;
begin
  Result := 0;
  PortInDQuery.paramA := APort;
  if RZDriver_IOCTRL (PortInDQuery) then Result := PortInDQuery.RESULT;
end;

{
  write byte to port
  out dx, al
}
procedure PortOutB (APort: WORD; AValue: BYTE);
begin
  PortOutBQuery.paramA := APort;
  PortOutBQuery.paramB := AValue;
  RZDriver_IOCTRL (PortOutBQuery);
end;

{
  write word to port
  out dx, ax
}
procedure PortOutW (APort: WORD; AValue: WORD);
begin
  PortOutWQuery.paramA := APort;
  PortOutWQuery.paramB := AValue;
  RZDriver_IOCTRL (PortOutWQuery);
end;

{
  write dword to port
  out dx, eax
}
procedure PortOutD (APort: WORD; AValue: DWORD);
begin
  PortOutDQuery.paramA := APort;
  PortOutDQuery.paramB := AValue;
  RZDriver_IOCTRL (PortOutDQuery);
end;

{
}
procedure SpeakerOn (ATone: WORD);
var
  VPORT : BYTE;
begin
  VPORT := PortInB ($61);
  PortOutB ($61, VPORT or $03);
  PortOutB ($43, $B6);
  PortOutB ($42, LO(ATone));
  PortOutB ($42, HI(ATone));
end;

{
}
procedure SpeakerOff;
var
  VPORT : BYTE;
begin
  VPORT := PortInB ($61);
  PortOutB ($61, VPORT and $FC);
end;

{
}
procedure PlaySound (AFrequency: WORD; ADelay: DWORD);
var
  VTone : WORD;
begin
  if AFrequency > 0 then begin
    VTone := Round (1193180.0 / AFrequency);
    SpeakerOn (VTone);
    Sleep (ADelay);
    SpeakerOff;
  end;
end;

end.
