
; FASM SOURCE FILE
; http://flatassembler.net

; Ring-0 Procedure Call & Direct Port I/O TEST (32-BIT)
; main project file

; written by @L.CHEMIST (Andrey A. Meshkov)
; http://maalchemist.narod.ru
; maalchemist@yandex.ru
; maalchemist@gmail.com

; ------------------------------------------------------------------------------

format PE GUI 4.0
entry zentry

DEFINE    _X64 FALSE
DEFINE  _WIN64 FALSE
DEFINE UNICODE FALSE

; ------------------------------------------------------------------------------

MAX_PATH = 260

SC_MANAGER_ALL_ACCESS = 0x000F003F
SERVICE_ALL_ACCESS    = 0x000F01FF
SERVICE_KERNEL_DRIVER = 0x00000001
SERVICE_DEMAND_START  = 0x00000003
SERVICE_ERROR_NORMAL  = 0x00000001
SERVICE_CONTROL_STOP  = 0x00000001
SERVICE_RUNNING       = 0x00000004

; ------------------------------------------------------------------------------

include 'TEST32.INC'

; ------------------------------------------------------------------------------

struct SERVICE_STATUS
    dwServiceType             dd ?
    dwCurrentState            dd ?
    dwControlsAccepted        dd ?
    dwWin32ExitCode           dd ?
    dwServiceSpecificExitCode dd ?
    dwCheckPoint              dd ?
    dwWaitHint                dd ?
ends

; ------------------------------------------------------------------------------

zentry:
;       sub     rsp, 8

        invoke  VirtualAlloc, 0, MAX_PATH, MEM_COMMIT, PAGE_READWRITE
        cmp     eax, 0
        je     .leave
        mov     [DrvPath], eax

        ; install driver
        invoke  GetFullPathName, DrvFilename, MAX_PATH, eax, DrvFile

        invoke  OpenSCManager, 0, 0, SC_MANAGER_ALL_ACCESS
        cmp     eax, 0
        je     .leave
        mov     [Manager], eax
        invoke  OpenService, eax, Drvname, SERVICE_ALL_ACCESS
        cmp     eax, 0
        jne     @F

        invoke  CreateService, [Manager], Drvname, Drvname,\
                SERVICE_ALL_ACCESS, SERVICE_KERNEL_DRIVER, SERVICE_DEMAND_START, SERVICE_ERROR_NORMAL,\
                [DrvPath], 0, 0, 0, 0, 0
        cmp     eax, 0
        je     .leave
    @@:
        mov     [Service], eax
        invoke  QueryServiceStatus, eax, Status
        cmp     eax, 0
        je     .leave
        cmp     [Status + SERVICE_STATUS.dwCurrentState], SERVICE_RUNNING
        je      @F

        invoke  StartService, [Service], 0, Vectors
        cmp     eax, 0
        jnz     @F
        procall svc_error_message, Buffer
        invoke  MessageBox, 0, Buffer, Error, 0
    @@:
        invoke  CreateFile, DrvDevice, GENERIC_WRITE or GENERIC_READ, FILE_SHARE_READ or FILE_SHARE_WRITE, 0, OPEN_EXISTING, 0, 0
        cmp     eax, INVALID_HANDLE_VALUE
        je     .leave
        mov     [Driver], eax

        ; do beep
        mov     eax, RZ_DRIVER_QUERY_PROC_NOARGS
        mov     dword [Query + RZDriverQuery.IOCODE], eax
        lea     eax, [beep]
        mov     dword [Query + RZDriverQuery.PROC], eax
        invoke  WriteFile, [Driver], Query, sizeof.RZDriverQuery, Bytes, 0

        ; uninstall driver
    .leave:
        mov     edi, [Driver]
        cmp     edi, INVALID_HANDLE_VALUE
        je      @F
        invoke  CloseHandle, edi
    @@:
        mov     esi, [Service]
        cmp     esi, 0
        je      @F
        invoke  ControlService, esi, SERVICE_CONTROL_STOP, Status
        invoke  DeleteService, esi
        invoke  CloseServiceHandle, esi
    @@:
        mov     edi, [Manager]
        cmp     edi, 0
        je      @F
        invoke  CloseServiceHandle, edi
    @@:
        mov     edi, [DrvPath]
        cmp     edi, 0
        je      @F
        invoke  VirtualFree, edi, MAX_PATH, MEM_RELEASE
    @@:

        ; go home
        invoke  ExitProcess, 0

; ------------------------------------------------------------------------------

beep:
        mov     al, 0B6h
        out     43h, al
        in      al, 61h
        or      al, 3
        out     61h, al

        mov     ax, 600
        mov     ecx, 30000
    @MUSIC:
        push    ecx

        out     42h, al
        mov     al, ah
        out     42h, al

        mov     ecx, 3000
    @DELAY:
        dec     ecx
        jnz     @DELAY
;       loop    @DELAY
        pop     ecx
        dec     ecx
        jnz     @MUSIC
;       loop    @MUSIC

        and     al, 11111100b
        out     61h, al
        retn

; ------------------------------------------------------------------------------

beeps:
        cli
        ; speaker ON
        in      al, 61h
        or      al, 11b
        out     61h, al
        sti

        mov     ecx, 18000000h
    @@: dec     ecx
        jnz     @B
;   @@: loop    @B

        cli
        ; speaker OFF
        in      al, 61h
        and     al, 11111100b
        out     61h, al
        sti
        retn

; ------------------------------------------------------------------------------

; [OUT] ERROR message buffer
proc svc_error_message, AERROR

;       mov     [AERROR], rcx
;       mov     [], rdx
;       mov     [], r8
;       mov     [], r9

    .ENTER:
;       test    rsp, 0x0F                       ; 16-byte stack align job
;       jnz     @F
;       push    rax
;     @@:
        push    ebx

        xor     eax, eax
        cmp     [AERROR], eax
        je     .EXIT

        ; DWORD GetLastError()
        invoke  GetLastError
;       and     rax, [MASK_0xFFFFFFFF]
        ; DWORD FormatMessage()
        invoke  FormatMessage, FORMAT_MESSAGE_FROM_SYSTEM, NULL, eax, 0, [AERROR], 1024, NULL
;       and     rax, [MASK_0xFFFFFFFF]
        test    eax, eax
        jz     .EXIT

        mov     ebx, [AERROR]
    .RTRIM:
        cmp     byte [ebx + eax], 32
        ja     .EXIT
        mov     byte [ebx + eax], 0
        dec     eax
        jnz    .RTRIM

    .EXIT:
        pop     ebx
        ret
endp

; ------------------------------------------------------------------------------

MASK_0xFFFFFFFF QWORD 0xFFFFFFFF

Error       db 'ERROR',0
Drvname     db 'RZ32',0
DrvFilename db 'RZ32.SYS',0
DrvDevice   db '\\.\RZ32',0
DrvPath     dd 0
Manager     dd 0
Service     dd 0
Driver      dd INVALID_HANDLE_VALUE

; ------------------------------------------------------------------------------

DrvFile dd ?
Vectors dd ?
Status  SERVICE_STATUS
Bytes   dd ?
Query   RZDriverQuery
Buffer  rb 1024

; ------------------------------------------------------------------------------

section '.idata' import data readable writeable

library kernel,'KERNEL32.DLL',\
        advapi,'ADVAPI32.DLL',\
        user,'USER32.DLL'

import  kernel,\
        VirtualAlloc,'VirtualAlloc',\
        VirtualFree,'VirtualFree',\
        CreateFile,'CreateFileA',\
        ReadFile,'ReadFile',\
        WriteFile,'WriteFile',\
        CloseHandle,'CloseHandle',\
        GetFullPathName,'GetFullPathNameA',\
        GetModuleHandle,'GetModuleHandleA',\
        GetLastError,'GetLastError',\
        FormatMessage,'FormatMessageA',\
        ExitProcess,'ExitProcess'

import  advapi,\
        OpenSCManager,'OpenSCManagerA',\
        CreateService,'CreateServiceA',\
        OpenService,'OpenServiceA',\
        QueryServiceStatus,'QueryServiceStatus',\
        StartService,'StartServiceA',\
        DeleteService,'DeleteService',\
        CloseServiceHandle,'CloseServiceHandle',\
        ControlService,'ControlService'

import  user,\
        MessageBox,'MessageBoxA'

; ------------------------------------------------------------------------------
