
; FASM SOURCE FILE
; http://flatassembler.net

; Ring-0 Procedure Call & Direct Port I/O TEST (64-BIT)
; main project file

; written by @L.CHEMIST (Andrey A. Meshkov)
; http://maalchemist.narod.ru
; maalchemist@yandex.ru
; maalchemist@gmail.com

; ------------------------------------------------------------------------------

format PE64 GUI 5.0
entry zentry

DEFINE    _X64  TRUE
DEFINE  _WIN64  TRUE
DEFINE UNICODE FALSE

; ------------------------------------------------------------------------------

MAX_PATH = 260

SC_MANAGER_ALL_ACCESS = 0x000F003F
SERVICE_ALL_ACCESS    = 0x000F01FF
SERVICE_KERNEL_DRIVER = 0x00000001
SERVICE_DEMAND_START  = 0x00000003
SERVICE_ERROR_NORMAL  = 0x00000001
SERVICE_CONTROL_STOP  = 0x00000001
SERVICE_RUNNING       = 0x00000004

; ------------------------------------------------------------------------------

include 'TEST64.INC'

; ------------------------------------------------------------------------------

struct SERVICE_STATUS
    dwServiceType             dd ?
    dwCurrentState            dd ?
    dwControlsAccepted        dd ?
    dwWin32ExitCode           dd ?
    dwServiceSpecificExitCode dd ?
    dwCheckPoint              dd ?
    dwWaitHint                dd ?
ends

; ------------------------------------------------------------------------------

zentry:
        sub     rsp, 8

        invoke  VirtualAlloc, 0, MAX_PATH, MEM_COMMIT, PAGE_READWRITE
        cmp     rax, 0
        je     .leave
        mov     [DrvPath], rax

        ; install driver
        lea     rcx, [DrvFilename]
        lea     r9, [DrvFile]
        invoke  GetFullPathName, rcx, MAX_PATH, eax, r9

;       invoke  MessageBox, 0, [DrvPath], [DrvFile], 0

        invoke  OpenSCManager, 0, 0, SC_MANAGER_ALL_ACCESS
        cmp     rax, 0
        je     .leave
        mov     [Manager], rax
        lea     rdx, [Drvname]
        invoke  OpenService, rax, rdx, SERVICE_ALL_ACCESS
        cmp     eax, 0
        jne     @F

        lea     rdx, [Drvname]
        invoke  CreateService, [Manager], rdx, rdx,\
                SERVICE_ALL_ACCESS, SERVICE_KERNEL_DRIVER, SERVICE_DEMAND_START, SERVICE_ERROR_NORMAL,\
                [DrvPath], 0, 0, 0, 0, 0
        cmp     rax, 0
        je     .leave
    @@:
        mov     [Service], rax
        lea     rdi, [Status]
        invoke  QueryServiceStatus, rax, rdi
        cmp     eax, 0
        je     .leave
        cmp     [rdi + SERVICE_STATUS.dwCurrentState], SERVICE_RUNNING
        je      @F

        lea     r8, [Vectors]
        invoke  StartService, [Service], 0, r8
        cmp     eax, 0
        jnz     @F
        lea     rcx, [Buffer]
        procall svc_error_message, rcx
        lea     rcx, [Buffer]
        lea     rdx, [Error]
        invoke  MessageBox, 0, rcx, rdx, 0
    @@:
        lea     rcx, [DrvDevice]
        invoke  CreateFile, rcx, GENERIC_WRITE or GENERIC_READ, FILE_SHARE_READ or FILE_SHARE_WRITE, 0, OPEN_EXISTING, 0, 0
        cmp     rax, INVALID_HANDLE_VALUE
        je     .leave
        mov     [Driver], rax

        ; do beep
        lea     rdx, [Query]
        mov     rax, RZ_DRIVER_QUERY_PROC_NOARGS
        mov     qword [rdx + RZDriverQuery.IOCODE], rax
        lea     rax, [beep]
        mov     qword [rdx + RZDriverQuery.PROC], rax
        lea     r9, [Bytes]
        invoke  WriteFile, [Driver], rdx, sizeof.RZDriverQuery, r9, 0

        ; uninstall driver
    .leave:
        mov     rdi, [Driver]
        cmp     rdi, INVALID_HANDLE_VALUE
        je      @F
        invoke  CloseHandle, rdi
    @@:
        mov     rsi, [Service]
        cmp     rsi, 0
        je      @F
        lea     r8, [Status]
        invoke  ControlService, rsi, SERVICE_CONTROL_STOP, r8
        invoke  DeleteService, rsi
        invoke  CloseServiceHandle, rsi
    @@:
        mov     rdi, [Manager]
        cmp     rdi, 0
        je      @F
        invoke  CloseServiceHandle, rdi
    @@:
        mov     rdi, [DrvPath]
        cmp     rdi, 0
        je      @F
        invoke  VirtualFree, rdi, MAX_PATH, MEM_RELEASE
    @@:

        ; go home
        invoke  ExitProcess, 0

; ------------------------------------------------------------------------------

beep:
        mov     al, 0B6h
        out     43h, al
        in      al, 61h
        or      al, 3
        out     61h, al

        mov     ax, 600
        mov     rcx, 30000
    @MUSIC:
        push    rcx

        out     42h, al
        mov     al, ah
        out     42h, al

        mov     rcx, 3000
    @DELAY:
        dec     ecx
        jnz     @DELAY
;       loop    @DELAY
        pop     rcx
        dec     ecx
        jnz     @MUSIC
;       loop    @MUSIC

        and     al, 11111100b
        out     61h, al
        retn

; ------------------------------------------------------------------------------

beeps:
        cli
        ; speaker ON
        in      al, 61h
        or      al, 11b
        out     61h, al
        sti

        mov     ecx, 18000000h
    @@: dec     ecx
        jnz     @B
;   @@: loop    @B

        cli
        ; speaker OFF
        in      al, 61h
        and     al, 11111100b
        out     61h, al
        sti
        retn

; ------------------------------------------------------------------------------

; [OUT] ERROR message buffer
proc svc_error_message, AERROR

        mov     [AERROR], rcx
;       mov     [], rdx
;       mov     [], r8
;       mov     [], r9

    .ENTER:
        test    rsp, 0x0F                       ; 16-byte stack align job
        jnz     @F
        push    rax
      @@:
        push    rbx

        xor     rax, rax
        cmp     [AERROR], rax
        je     .EXIT

        ; DWORD GetLastError()
        invoke  GetLastError
        and     rax, [MASK_0xFFFFFFFF]
        ; DWORD FormatMessage()
        invoke  FormatMessage, FORMAT_MESSAGE_FROM_SYSTEM, NULL, rax, 0, [AERROR], 1024, NULL
        and     rax, [MASK_0xFFFFFFFF]
        test    rax, rax
        jz     .EXIT

        mov     rbx, [AERROR]
    .RTRIM:
        cmp     byte [rbx + rax], 32
        ja     .EXIT
        mov     byte [rbx + rax], 0
        dec     rax
        jnz    .RTRIM

    .EXIT:
        pop     rbx
        ret
endp

; ------------------------------------------------------------------------------

MASK_0xFFFFFFFF QWORD 0xFFFFFFFF

Error       db 'ERROR',0
Drvname     db 'RZ64',0
DrvFilename db 'RZ64.SYS',0
DrvDevice   db '\\.\RZ64',0
DrvPath     dq 0
Manager     dq 0
Service     dq 0
Driver      dq INVALID_HANDLE_VALUE

; ------------------------------------------------------------------------------

DrvFile dq ?
Vectors dq ?
Status  SERVICE_STATUS
Bytes   dq ?
Query   RZDriverQuery
Buffer  rb 1024

; ------------------------------------------------------------------------------

section '.idata' import data readable writeable

library kernel,'KERNEL32.DLL',\
        advapi,'ADVAPI32.DLL',\
        user,'USER32.DLL'

import  kernel,\
        VirtualAlloc,'VirtualAlloc',\
        VirtualFree,'VirtualFree',\
        CreateFile,'CreateFileA',\
        ReadFile,'ReadFile',\
        WriteFile,'WriteFile',\
        CloseHandle,'CloseHandle',\
        GetFullPathName,'GetFullPathNameA',\
        GetModuleHandle,'GetModuleHandleA',\
        GetLastError,'GetLastError',\
        FormatMessage,'FormatMessageA',\
        ExitProcess,'ExitProcess'

import  advapi,\
        OpenSCManager,'OpenSCManagerA',\
        CreateService,'CreateServiceA',\
        OpenService,'OpenServiceA',\
        QueryServiceStatus,'QueryServiceStatus',\
        StartService,'StartServiceA',\
        DeleteService,'DeleteService',\
        CloseServiceHandle,'CloseServiceHandle',\
        ControlService,'ControlService'

import  user,\
        MessageBox,'MessageBoxA'

; ------------------------------------------------------------------------------
