
; FASM SOURCE FILE
; http://flatassembler.net

; Ring-0 Procedure Call & Direct Port I/O (32-BIT)
; .text section

; written by @L.CHEMIST (Andrey A. Meshkov)
; http://maalchemist.narod.ru
; maalchemist@yandex.ru
; maalchemist@gmail.com

; ------------------------------------------------------------------------------

;  NTSTATUS
;  DriverEntry (
;    IN PDRIVER_OBJECT   lpDriverObject
;    IN PUNICODE_STRING  lpusRegistryPath
;  )

align 10h
proc dzentry, lpDriverObject, lpusRegistryPath

    locals
       .USD     UNICODE_STRING ?
       .USSL    UNICODE_STRING ?
       .USE     UNICODE_STRING ?
    endl

   .enter:
        push    ebx esi edi

        lea     ecx, [.USD]
        lea     edx, [usDevice]
        invoke  RtlInitUnicodeString, ecx, edx

        ; create and initialize device object
        lea     esi, [.USD]
        ; NTSTATUS IoCreateDevice()
        invoke  IoCreateDevice, [lpDriverObject], RZ_DRIVER_BUFFER_SIZE, esi,\
                FILE_DEVICE_UNKNOWN, 0, FALSE, DeviceObject
        cmp     eax, STATUS_SUCCESS
        jne    .return ; break on error

        ; assign device extension variable
        mov     eax, [DeviceObject]
        mov     eax, [eax + V_DEVICE_OBJECT.DeviceExtension]
        mov     [DeviceExtension], eax

        lea     ecx, [.USSL]
        lea     edx, [usSymbolicLink]
        invoke  RtlInitUnicodeString, ecx, edx

        ; create symbolic link to the user-visible name
        ; NTSTATUS IoCreateSymbolicLink()
        lea     ecx, [.USSL]
        lea     edx, [.USD]
        invoke  IoCreateSymbolicLink, ecx, edx

        ; save status
        mov     ebx, eax

        ; check result
        cmp     eax, STATUS_SUCCESS
        je     .success

        ; delete device object if not successful
        invoke  IoDeleteDevice, [DeviceObject]
        jmp    .return

   .success: ; continue on success
        ; load structure to point to IRP handlers
        mov     eax, [lpDriverObject]
        lea     edx, [DriverUnload]
        mov     [eax + V_DRIVER_OBJECT.DriverUnload], edx
        lea     edx, [DispatchCreateClose]
        mov     [eax + V_DRIVER_OBJECT.MajorFunction + IRP_MJ_CREATE_OFFSET], edx
        mov     [eax + V_DRIVER_OBJECT.MajorFunction + IRP_MJ_CLOSE_OFFSET], edx
        lea     edx, [DispatchReadWrite]
        mov     [eax + V_DRIVER_OBJECT.MajorFunction + IRP_MJ_READ_OFFSET], edx
        mov     [eax + V_DRIVER_OBJECT.MajorFunction + IRP_MJ_WRITE_OFFSET], edx

        ; create "create process event" for user-mode processes to monitor & clear it out
        lea     edi, [.USE]
        lea     edx, [usProcessEvent]
        invoke  RtlInitUnicodeString, edi, edx
        invoke  IoCreateSynchronizationEvent, edi, HProcessEvent
;       invoke  IoCreateNotificationEvent, edi, HProcessEvent
        mov     [KProcessEvent], eax
        invoke  KeClearEvent, [KProcessEvent]

        lea     edi, [.USE]
        lea     edx, [usCPCEvent]
        invoke  RtlInitUnicodeString, edi, edx
        invoke  IoCreateNotificationEvent, edi, HCPCEvent
        mov     [KCPCEvent], eax
        invoke  KeSetEvent, [KCPCEvent], 0, FALSE

        lea     edi, [.USE]
        lea     edx, [usCPGEvent]
        invoke  RtlInitUnicodeString, edi, edx
        invoke  IoCreateNotificationEvent, edi, HCPGEvent
        mov     [KCPGEvent], eax
        invoke  KeSetEvent, [KCPGEvent], 0, FALSE

        ; assign result
        mov     eax, ebx
   .return:
        pop     edi esi ebx
        ret
endp

; ------------------------------------------------------------------------------

;  VOID
;  DriverUnload (
;    IN PDRIVER_OBJECT  lpDriverObject
;  )

align 10h
proc DriverUnload, lpDriverObject

    locals
       .USSL    UNICODE_STRING ?
    endl

   .enter:
        ; restore the call back routine, thus givinig chance to the
        ; user mode application to unload dynamically the driver
        cmp     [CP_Activated], 0
        je      @F
        invoke  PsSetCreateProcessNotifyRoutine, CreateProcessCallback, TRUE
    @@:

        ; close "create process event"
        cmp     [HProcessEvent], 0
        je      @F
        invoke  ZwClose, [HProcessEvent]
    @@:

        cmp     [HCPCEvent], 0
        je      @F
        invoke  ZwClose, [HCPCEvent]
    @@:

        cmp     [HCPGEvent], 0
        je      @F
        invoke  ZwClose, [HCPGEvent]
    @@:

        lea     ecx, [.USSL]
        lea     edx, [usSymbolicLink]
        invoke  RtlInitUnicodeString, ecx, edx

        ; delete symbolic link to the user-visible name
        ; NTSTATUS IoDeleteSymbolicLink()
        lea     ecx, [.USSL]
        invoke  IoDeleteSymbolicLink, ecx

        ; delete device object
        ; VOID IoDeleteDevice()
        mov     ecx, [lpDriverObject]
        mov     ecx, [ecx + V_DRIVER_OBJECT.DeviceObject]
        invoke  IoDeleteDevice, ecx

   .return:
        ret
endp

; ------------------------------------------------------------------------------

;  NTSTATUS
;  DispatchCreateClose (
;    IN PDEVICE_OBJECT  lpDeviceObject,
;    IN PIRP            lpIrp
;  )

align 10h
proc DispatchCreateClose, lpDeviceObject, lpIrp

   .enter:
        mov     eax, [lpIrp]
        mov     [eax + V_IRP.IoStatus.Status], STATUS_SUCCESS
        mov     [eax + V_IRP.IoStatus.Information], 0
        ; VOID IoCompleteRequest()
        invoke  IoCompleteRequest, [lpIrp], IO_NO_INCREMENT

        mov     eax, STATUS_SUCCESS
   .return:
        ret
endp

; ------------------------------------------------------------------------------

;  NTSTATUS
;  DispatchReadWrite (
;    IN PDEVICE_OBJECT  lpDeviceObject,
;    IN PIRP            lpIrp
;  )

align 10h
proc DispatchReadWrite, lpDeviceObject, lpIrp

   .enter:
        push    ebx esi edi

        mov     ebx, STATUS_UNSUCCESSFUL                                ; EBX = current status

        mov     edi, [lpIrp]                                            ; EDI = PIRP
        mov     [edi + V_IRP.IoStatus.Information], 0
        mov     esi, [edi + V_IRP.Tail.Overlay.CurrentStackLocation]    ; ESI = PIO_STACK_LOCATION

        mov     ecx, [esi + V_IO_STACK_LOCATION.Parameters.Write.Length]
        cmp     ecx, RZ_DRIVER_BUFFER_SIZE
        ja     .leave

   .RZDriverQuery:
        mov     ebx, STATUS_NOT_IMPLEMENTED
        mov     esi, [edi + V_IRP.UserBuffer]                           ; ESI = PR0DriverQuery
        mov     dword [esi + V_RZDriverQuery.RESULT], 0
        mov     ecx, dword [esi + V_RZDriverQuery.IOCODE]               ; ECX = user I/O code

   .TEST.procs:
        cmp     ecx, RZ_DRIVER_QUERY_PROC_NOARGS
        je     .proc.noargs
        cmp     ecx, RZ_DRIVER_QUERY_PROC_NOARGS_IFRAME
        je     .proc.noargs.iframe
        cmp     ecx, RZ_DRIVER_QUERY_PROC_REGISTER
        je     .proc.register
        cmp     ecx, RZ_DRIVER_QUERY_PROC_REGISTER_IFRAME
        je     .proc.register.iframe
        cmp     ecx, RZ_DRIVER_QUERY_PROC_REGISTER_2P
        je     .proc.register.2p
        cmp     ecx, RZ_DRIVER_QUERY_PROC_REGISTER_2P_IFRAME
        je     .proc.register.2p.iframe
        cmp     ecx, RZ_DRIVER_QUERY_PROC_STDCALL
        je     .proc.stdcall
        cmp     ecx, RZ_DRIVER_QUERY_PROC_STDCALL_IFRAME
        je     .proc.stdcall.iframe
        cmp     ecx, RZ_DRIVER_QUERY_PROC_STDCALL_2P
        je     .proc.stdcall.2p
        cmp     ecx, RZ_DRIVER_QUERY_PROC_STDCALL_2P_IFRAME
        je     .proc.stdcall.2p.iframe
        cmp     ecx, RZ_DRIVER_QUERY_PROC_CDECL
        je     .proc.cdecl
        cmp     ecx, RZ_DRIVER_QUERY_PROC_CDECL_IFRAME
        je     .proc.cdecl.iframe
        cmp     ecx, RZ_DRIVER_QUERY_PROC_CDECL_2P
        je     .proc.cdecl.2p
        cmp     ecx, RZ_DRIVER_QUERY_PROC_CDECL_2P_IFRAME
        je     .proc.cdecl.2p.iframe
   .TEST.IO:
        cmp     ecx, RZ_DRIVER_QUERY_PORT_IN_BYTE
        je     .port.in.byte
        cmp     ecx, RZ_DRIVER_QUERY_PORT_IN_WORD
        je     .port.in.word
        cmp     ecx, RZ_DRIVER_QUERY_PORT_IN_DWORD
        je     .port.in.dword
        cmp     ecx, RZ_DRIVER_QUERY_PORT_OUT_BYTE
        je     .port.out.byte
        cmp     ecx, RZ_DRIVER_QUERY_PORT_OUT_WORD
        je     .port.out.word
        cmp     ecx, RZ_DRIVER_QUERY_PORT_OUT_DWORD
        je     .port.out.dword
   .TEST.map:
        cmp     ecx, RZ_DRIVER_QUERY_MAP_IO_SPACE
        je     .map.io.space
        cmp     ecx, RZ_DRIVER_QUERY_MAP_IO_SPACE_IFRAME
        je     .map.io.space.iframe
        cmp     ecx, RZ_DRIVER_QUERY_UNMAP_IO_SPACE
        je     .unmap.io.space
        cmp     ecx, RZ_DRIVER_QUERY_UNMAP_IO_SPACE_IFRAME
        je     .unmap.io.space.iframe
   .TEST.cp:
        cmp     ecx, RZ_DRIVER_QUERY_CP_SET
        je     .cp.set
        cmp     ecx, RZ_DRIVER_QUERY_CP_SET_IFRAME
        je     .cp.set.iframe
        cmp     ecx, RZ_DRIVER_QUERY_CP_RET
        je     .cp.ret
        cmp     ecx, RZ_DRIVER_QUERY_CP_RET_IFRAME
        je     .cp.ret.iframe
        cmp     ecx, RZ_DRIVER_QUERY_CP_GET
        je     .cp.get
        cmp     ecx, RZ_DRIVER_QUERY_CP_GET_IFRAME
        je     .cp.get.iframe

        jmp    .leave

   .proc.noargs:
        mov     eax, dword [esi + V_RZDriverQuery.PROC]
        or      eax, eax
        jz     .leave
        call    eax
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .proc.noargs.iframe:
        mov     eax, dword [esi + V_RZDriverQuery.PROC]
        or      eax, eax
        jz     .leave
        cli
        call    eax
        mov     ebx, STATUS_SUCCESS
        sti
        jmp    .leave

   .proc.register:
        mov     ecx, dword [esi + V_RZDriverQuery.PROC]
        or      ecx, ecx
        jz     .leave
        mov     eax, dword [esi + V_RZDriverQuery.paramA]
        call    ecx
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .proc.register.iframe:
        mov     ecx, dword [esi + V_RZDriverQuery.PROC]
        or      ecx, ecx
        jz     .leave
        cli
        mov     eax, dword [esi + V_RZDriverQuery.paramA]
        call    ecx
        mov     ebx, STATUS_SUCCESS
        sti
        jmp    .leave

   .proc.register.2p:
        mov     ecx, dword [esi + V_RZDriverQuery.PROC]
        or      ecx, ecx
        jz     .leave
        mov     eax, dword [esi + V_RZDriverQuery.paramA]
        mov     edx, dword [esi + V_RZDriverQuery.paramB]
        call    ecx
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .proc.register.2p.iframe:
        mov     ecx, dword [esi + V_RZDriverQuery.PROC]
        or      ecx, ecx
        jz     .leave
        cli
        mov     eax, dword [esi + V_RZDriverQuery.paramA]
        mov     edx, dword [esi + V_RZDriverQuery.paramB]
        call    ecx
        mov     ebx, STATUS_SUCCESS
        sti
        jmp    .leave

   .proc.stdcall:
        mov     eax, dword [esi + V_RZDriverQuery.PROC]
        or      eax, eax
        jz     .leave
        push    dword [esi + V_RZDriverQuery.paramA]
        call    eax
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .proc.stdcall.iframe:
        mov     eax, dword [esi + V_RZDriverQuery.PROC]
        or      eax, eax
        jz     .leave
        cli
        push    dword [esi + V_RZDriverQuery.paramA]
        call    eax
        mov     ebx, STATUS_SUCCESS
        sti
        jmp    .leave

   .proc.stdcall.2p:
        mov     eax, dword [esi + V_RZDriverQuery.PROC]
        or      eax, eax
        jz     .leave
        push    dword [esi + V_RZDriverQuery.paramB]
        push    dword [esi + V_RZDriverQuery.paramA]
        call    eax
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .proc.stdcall.2p.iframe:
        mov     eax, dword [esi + V_RZDriverQuery.PROC]
        or      eax, eax
        jz     .leave
        cli
        push    dword [esi + V_RZDriverQuery.paramB]
        push    dword [esi + V_RZDriverQuery.paramA]
        call    eax
        mov     ebx, STATUS_SUCCESS
        sti
        jmp    .leave

   .proc.cdecl:
        mov     eax, dword [esi + V_RZDriverQuery.PROC]
        or      eax, eax
        jz     .leave
        push    dword [esi + V_RZDriverQuery.paramA]
        call    eax
        add     esp, 04
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .proc.cdecl.iframe:
        mov     eax, dword [esi + V_RZDriverQuery.PROC]
        or      eax, eax
        jz     .leave
        cli
        push    dword [esi + V_RZDriverQuery.paramA]
        call    eax
        add     esp, 04
        mov     ebx, STATUS_SUCCESS
        sti
        jmp    .leave

   .proc.cdecl.2p:
        mov     eax, dword [esi + V_RZDriverQuery.PROC]
        or      eax, eax
        jz     .leave
        push    dword [esi + V_RZDriverQuery.paramB]
        push    dword [esi + V_RZDriverQuery.paramA]
        call    eax
        add     esp, 08
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .proc.cdecl.2p.iframe:
        mov     eax, dword [esi + V_RZDriverQuery.PROC]
        or      eax, eax
        jz     .leave
        cli
        push    dword [esi + V_RZDriverQuery.paramB]
        push    dword [esi + V_RZDriverQuery.paramA]
        call    eax
        add     esp, 08
        mov     ebx, STATUS_SUCCESS
        sti
        jmp    .leave

   .port.in.byte:
        mov     dx, word [esi + V_RZDriverQuery.paramA]
        in      al, dx
        mov     byte [esi + V_RZDriverQuery.RESULT], al
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .port.in.word:
        mov     dx, word [esi + V_RZDriverQuery.paramA]
        in      ax, dx
        mov     word [esi + V_RZDriverQuery.RESULT], ax
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .port.in.dword:
        mov     dx, word [esi + V_RZDriverQuery.paramA]
        in      eax, dx
        mov     dword [esi + V_RZDriverQuery.RESULT], eax
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .port.out.byte:
        mov     dx, word [esi + V_RZDriverQuery.paramA]
        mov     al, byte [esi + V_RZDriverQuery.paramB]
        out     dx, al
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .port.out.word:
        mov     dx, word [esi + V_RZDriverQuery.paramA]
        mov     ax, word [esi + V_RZDriverQuery.paramB]
        out     dx, ax
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .port.out.dword:
        mov     dx, word [esi + V_RZDriverQuery.paramA]
        mov     eax, dword [esi + V_RZDriverQuery.paramB]
        out     dx, eax
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .map.io.space:
        procall mapiospace, esi
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .map.io.space.iframe:
        cli
        procall mapiospace, esi
        mov     ebx, STATUS_SUCCESS
        sti
        jmp    .leave

   .unmap.io.space:
        procall unmapiospace, esi
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .unmap.io.space.iframe:
        cli
        procall unmapiospace, esi
        mov     ebx, STATUS_SUCCESS
        sti
        jmp    .leave

   .cp.set:
        procall CreateProcessSet, esi
        jmp    .leave

   .cp.set.iframe:
        cli
        procall CreateProcessSet, esi
        sti
        jmp    .leave

   .cp.ret:
        procall CreateProcessRet, esi
        jmp    .leave

   .cp.ret.iframe:
        cli
        procall CreateProcessRet, esi
        sti
        jmp    .leave

   .cp.get:
        procall CreateProcessGet, esi
        jmp    .leave

   .cp.get.iframe:
        cli
        procall CreateProcessGet, esi
        sti
        jmp    .leave

   .leave:
        mov     [edi + V_IRP.IoStatus.Status], ebx
        ; VOID IoCompleteRequest()
        invoke  IoCompleteRequest, edi, IO_NO_INCREMENT

   .return:
        mov     eax, ebx
        pop     edi esi ebx
        ret
endp

; ------------------------------------------------------------------------------

; PVOID
; MmMapIoSpace(
;   IN PHYSICAL_ADDRESS     PhysicalAddress,
;   IN SIZE_T               NumberOfBytes,
;   IN MEMORY_CACHING_TYPE  CacheType
; );

; VOID
; MmUnmapIoSpace(
;   IN PVOID   BaseAddress,
;   IN SIZE_T  NumberOfBytes
; );

align 10h
proc mapiospace, lpQuery

   .enter:
        push    esi edi

        mov     esi, [lpQuery]
        or      esi, esi
        jz     .return

        mov     ecx, dword [esi + V_RZDriverIOQuery.SRC]        ; PhysicalAddress.LO
        or      ecx, ecx
        jz     .return
        mov     edi, dword [esi + V_RZDriverIOQuery.SRC + 4]    ; PhysicalAddress.HI
        mov     edx, dword [esi + V_RZDriverIOQuery.BYTES]      ; NumberOfBytes
        or      edx, edx
        jz     .return
        xor     eax, eax                                        ; CacheType :: MmNonCached (FALSE)
        invoke  MmMapIoSpace, ecx, edi, edx, eax
        mov     dword [esi + V_RZDriverIOQuery.RESULT], eax     ; Save virtual address
        or      eax, eax
        jz     .return

        mov     ecx, dword [esi + V_RZDriverIOQuery.DST]        ; Destination
        mov     edx, dword [esi + V_RZDriverIOQuery.RESULT]     ; Source
        mov     eax, dword [esi + V_RZDriverIOQuery.BYTES]      ; Length
        cinvoke memcpy, ecx, edx, eax

        mov     ecx, dword [esi + V_RZDriverIOQuery.IODATA]     ; Skip MmUnmapIoSpace if IODATA != 0
        or      ecx, ecx
        jnz    .return

        mov     ecx, dword [esi + V_RZDriverIOQuery.RESULT]     ; BaseAddress
        mov     edx, dword [esi + V_RZDriverIOQuery.BYTES]      ; NumberOfBytes
        invoke  MmUnmapIoSpace, ecx, edx

   .return:
        pop     edi esi
        ret
endp

; ------------------------------------------------------------------------------

; VOID
; MmUnmapIoSpace(
;   IN PVOID   BaseAddress,
;   IN SIZE_T  NumberOfBytes
; );

align 10h
proc unmapiospace, lpQuery

   .enter:
        push    esi edi

        mov     esi, [lpQuery]
        or      esi, esi
        jz     .return

        mov     ecx, dword [esi + V_RZDriverIOQuery.SRC]        ; BaseAddress
        or      ecx, ecx
        jz     .return
        mov     edx, dword [esi + V_RZDriverIOQuery.BYTES]      ; NumberOfBytes
        or      edx, edx
        jz     .return
        invoke  MmUnmapIoSpace, ecx, edx

   .return:
        pop     edi esi
        ret
endp

; ------------------------------------------------------------------------------

; NTSTATUS
; CreateProcessSet ()

align 10h
proc CreateProcessSet, lpQuery

   .enter:
        mov     ecx, [lpQuery]
        test    ecx, ecx
        jz     .return

        mov     eax, dword [ecx + V_RZDriverCPQuery.ProcessID]
        test    eax, eax
        jz      @F
        mov     dword [CProcessID], eax
    @@:

        cmp     [CP_Activated], 1
        je     .return

        invoke  PsSetCreateProcessNotifyRoutine, CreateProcessCallback, FALSE
        cmp     eax, STATUS_SUCCESS
        jne    .return ; break on error

        mov     [CP_Activated], 1
        jmp    .return

   .return:
        ret
endp

; ------------------------------------------------------------------------------

; NTSTATUS
; CreateProcessRet ()

align 10h
proc CreateProcessRet, lpQuery

   .enter:
        cmp     [CP_Activated], 0
        je     .return

        invoke  PsSetCreateProcessNotifyRoutine, CreateProcessCallback, TRUE
        cmp     eax, STATUS_SUCCESS
        jne    .return ; break on error

        mov     [CP_Activated], 0
        jmp    .return

   .return:
        ret
endp

; ------------------------------------------------------------------------------

align 10h
proc CreateProcessGet, lpQuery

   .enter:
        push    ebx esi edi

        mov     edi, [lpQuery]
        or      edi, edi
        jz     .return

        mov     esi, [DeviceExtension]
        or      esi, esi
        jz     .return

        invoke  KeWaitForSingleObject, [KCPGEvent], Executive, KernelMode, FALSE, NULL
        invoke  KeWaitForSingleObject, [KCPCEvent], Executive, KernelMode, FALSE, NULL
   .lock:
        invoke  KeClearEvent, [KCPGEvent]
        xor     ecx, ecx
        mov     cx, [CP_BuffDepth]
        or      cx, cx
        jz     .unlock
   .copy:
        mov     eax, dword [esi + V_RZDriverCPQuery.ParentID]
        mov     dword [edi + V_RZDriverCPQuery.ParentID], eax
        mov     eax, dword [esi + V_RZDriverCPQuery.ProcessID]
        mov     dword [edi + V_RZDriverCPQuery.ProcessID], eax
        mov     eax, dword [esi + V_RZDriverCPQuery.Create]
        mov     dword [edi + V_RZDriverCPQuery.Create], eax
        mov     eax, dword [esi + V_RZDriverCPQuery.RESULT]
        mov     dword [edi + V_RZDriverCPQuery.RESULT], eax
        mov     word [edi + V_RZDriverCPQuery.RESULT+4], cx
        mov     ax, [CP_DropCount]
        mov     word [edi + V_RZDriverCPQuery.RESULT+6], ax
   .shift:
        lea     esi, [esi + sizeof.RZDriverQuery]
        lea     edi, [edi + sizeof.RZDriverQuery]
        dec     [CP_BuffDepth]
        loop   .copy
   .unlock:
        invoke  KeSetEvent, [KCPGEvent], 0, FALSE

   .return:
        pop     edi esi ebx
        ret
endp

; ------------------------------------------------------------------------------

; VOID
; CreateProcessCallback (
;   IN HANDLE   hParentID,
;   IN HANDLE   hProcessID,
;   IN BOOLEAN  bCreate
; )

align 10h
proc CreateProcessCallback, hParentID, hProcessID, bCreate

   .enter:
        push    edi

        cmp     [CP_Activated], 0
        je     .return

        cmp     dword [CProcessID], 0
        je      @F
        mov     edx, [hProcessID]
        cmp     dword [CProcessID], edx
        je     .return
    @@:

        ; assign current values into device extension.
        ; user-mode apps will pick it up using DeviceIoControl calls.
        mov     edi, [DeviceExtension]
        or      edi, edi
        jz     .return

;       invoke  KeWaitForSingleObject, [KCPCEvent], Executive, KernelMode, FALSE, NULL
        invoke  KeWaitForSingleObject, [KCPGEvent], Executive, KernelMode, FALSE, NULL
   .lock:
        invoke  KeClearEvent, [KCPCEvent]
        inc     [CP_RecvCount]
        xor     eax, eax
        mov     ax, [CP_BuffDepth]
        cmp     ax, RZ_DRIVER_BUFFER_CAPACITY
        jb     .assign
        inc     [CP_DropCount]
        jmp    .unlock
   .assign:
        inc     [CP_BuffDepth]
        mov     ecx, sizeof.RZDriverQuery
        mul     ecx
        lea     edi, [edi + eax]
        mov     ecx, [hParentID]
        mov     dword [edi + V_RZDriverCPQuery.ParentID], ecx
        mov     ecx, [hProcessID]
        mov     dword [edi + V_RZDriverCPQuery.ProcessID], ecx
        mov     ecx, [bCreate]
        mov     dword [edi + V_RZDriverCPQuery.Create], ecx
        mov     ecx, [CP_RecvCount]
        mov     dword [edi + V_RZDriverCPQuery.RESULT], ecx
   .unlock:
        invoke  KeSetEvent, [KCPCEvent], 0, FALSE

   .event:
        invoke  KeReadStateEvent, [KCPCEvent]
        or      eax, eax
        jz     .event.OFF
        ; signal the event thus the user-mode apps listening will be aware
        ; that something interesting has happened.
        invoke  KeSetEvent, [KProcessEvent], 0, FALSE
;       invoke  KeClearEvent, [KProcessEvent]
   .event.OFF:

   .return:
        pop     edi
        ret
endp

; ------------------------------------------------------------------------------

align 10h
proc memcopy, Destination, Source, Length

   .enter:
        push    esi edi

        cld
        mov     edi, [Destination]
        mov     esi, [Source]

        mov     ecx, [Length]
        shr     ecx, 2
        rep     movsd

        mov     ecx, [Length]
        and     ecx, 3
        rep     movsb

   .return:
        pop     edi esi
        ret
endp

; ------------------------------------------------------------------------------
