
; FASM SOURCE FILE
; http://flatassembler.net

; Ring-0 Procedure Call & Direct Port I/O (64-BIT)
; .text section

; written by @L.CHEMIST (Andrey A. Meshkov)
; http://maalchemist.narod.ru
; maalchemist@yandex.ru
; maalchemist@gmail.com

; ------------------------------------------------------------------------------

;  NTSTATUS
;  DriverEntry (
;    IN PDRIVER_OBJECT   lpDriverObject
;    IN PUNICODE_STRING  lpusRegistryPath
;  )

align 10h
proc dzentry, lpDriverObject, lpusRegistryPath

    locals
       .USD     UNICODE_STRING ?
       .USSL    UNICODE_STRING ?
       .USE     UNICODE_STRING ?
    endl

        mov     [lpDriverObject], rcx
        mov     [lpusRegistryPath], rdx
;       mov     [], r8
;       mov     [], r9

   .enter:
        test    rsp, 0x0F                       ; 16-byte stack align job
        jnz     @F
        push    rax
      @@:
        push    rbx rsi rdi

        mov     rbx, rcx

        lea     rcx, [.USD]
        lea     rdx, [usDevice]
        invoke  RtlInitUnicodeString, rcx, rdx

        ; create and initialize device object
        lea     r8, [.USD]
        ; NTSTATUS IoCreateDevice()
        invoke  IoCreateDevice, [lpDriverObject], RZ_DRIVER_BUFFER_SIZE, r8,\
                FILE_DEVICE_UNKNOWN, 0, FALSE, DeviceObject
        cmp     eax, STATUS_SUCCESS
        jne    .return ; break on error

        ; assign device extension variable
        mov     rax, [DeviceObject]
        mov     rax, [rax + V_DEVICE_OBJECT.DeviceExtension]
        mov     [DeviceExtension], rax

        lea     rcx, [.USSL]
        lea     rdx, [usSymbolicLink]
        invoke  RtlInitUnicodeString, rcx, rdx

        ; create symbolic link to the user-visible name
        ; NTSTATUS IoCreateSymbolicLink()
        lea     rcx, [.USSL]
        lea     rdx, [.USD]
        invoke  IoCreateSymbolicLink, rcx, rdx

        ; save status
        mov     ebx, eax

        ; check result
        cmp     eax, STATUS_SUCCESS
        je     .success

        ; delete device object if not successful
        invoke  IoDeleteDevice, [DeviceObject]
        jmp    .return

   .success: ; continue on success
        ; load structure to point to IRP handlers
        mov     rax, [lpDriverObject]
        lea     rdx, [DriverUnload]
        mov     [rax + V_DRIVER_OBJECT.DriverUnload], rdx
        lea     rdx, [DispatchCreateClose]
        mov     [rax + V_DRIVER_OBJECT.MajorFunction + IRP_MJ_CREATE_OFFSET], rdx
        mov     [rax + V_DRIVER_OBJECT.MajorFunction + IRP_MJ_CLOSE_OFFSET], rdx
        lea     rdx, [DispatchReadWrite]
        mov     [rax + V_DRIVER_OBJECT.MajorFunction + IRP_MJ_READ_OFFSET], rdx
        mov     [rax + V_DRIVER_OBJECT.MajorFunction + IRP_MJ_WRITE_OFFSET], rdx

        ; create "create process event" for user-mode processes to monitor & clear it out
        lea     rdi, [.USE]
        lea     rdx, [usProcessEvent]
        invoke  RtlInitUnicodeString, rdi, rdx
        invoke  IoCreateSynchronizationEvent, rdi, HProcessEvent
;       invoke  IoCreateNotificationEvent, rdi, HProcessEvent
        mov     [KProcessEvent], rax
        invoke  KeClearEvent, [KProcessEvent]

        lea     rdi, [.USE]
        lea     rdx, [usCPCEvent]
        invoke  RtlInitUnicodeString, rdi, rdx
        invoke  IoCreateNotificationEvent, rdi, HCPCEvent
        mov     [KCPCEvent], rax
        invoke  KeSetEvent, [KCPCEvent], 0, FALSE

        lea     rdi, [.USE]
        lea     rdx, [usCPGEvent]
        invoke  RtlInitUnicodeString, rdi, rdx
        invoke  IoCreateNotificationEvent, rdi, HCPGEvent
        mov     [KCPGEvent], rax
        invoke  KeSetEvent, [KCPGEvent], 0, FALSE

        ; assign result
        mov     eax, ebx
   .return:
        pop     rdi rsi rbx
        ret
endp

; ------------------------------------------------------------------------------

;  VOID
;  DriverUnload (
;    IN PDRIVER_OBJECT  lpDriverObject
;  )

align 10h
proc DriverUnload, lpDriverObject

    locals
       .USSL    UNICODE_STRING ?
    endl

        mov     [lpDriverObject], rcx
;       mov     [], rdx
;       mov     [], r8
;       mov     [], r9

   .enter:
        test    rsp, 0x0F                       ; 16-byte stack align job
        jz      @F
        push    rax
      @@:

        ; restore the call back routine, thus givinig chance to the
        ; user mode application to unload dynamically the driver
        cmp     [CP_Activated], 0
        je      @F
        invoke  PsSetCreateProcessNotifyRoutine, CreateProcessCallback, TRUE
    @@:

        ; close "create process event"
        cmp     [HProcessEvent], 0
        je      @F
        invoke  ZwClose, [HProcessEvent]
    @@:

        cmp     [HCPCEvent], 0
        je      @F
        invoke  ZwClose, [HCPCEvent]
    @@:

        cmp     [HCPGEvent], 0
        je      @F
        invoke  ZwClose, [HCPGEvent]
    @@:

        lea     rcx, [.USSL]
        lea     rdx, [usSymbolicLink]
        invoke  RtlInitUnicodeString, rcx, rdx

        ; delete symbolic link to the user-visible name
        ; NTSTATUS IoDeleteSymbolicLink()
        lea     rcx, [.USSL]
        invoke  IoDeleteSymbolicLink, rcx

        ; delete device object
        ; VOID IoDeleteDevice()
        mov     rcx, [lpDriverObject]
        mov     rcx, [rcx + V_DRIVER_OBJECT.DeviceObject]
        invoke  IoDeleteDevice, rcx

   .return:
        ret
endp

; ------------------------------------------------------------------------------

;  NTSTATUS
;  DispatchCreateClose (
;    IN PDEVICE_OBJECT  lpDeviceObject,
;    IN PIRP            lpIrp
;  )

align 10h
proc DispatchCreateClose, lpDeviceObject, lpIrp

        mov     [lpDeviceObject], rcx
        mov     [lpIrp], rdx
;       mov     [], r8
;       mov     [], r9

   .enter:
        test    rsp, 0x0F                       ; 16-byte stack align job
        jz      @F
        push    rax
      @@:

        mov     rax, [lpIrp]
        mov     [rax + V_IRP.IoStatus.Status], STATUS_SUCCESS
        mov     [rax + V_IRP.IoStatus.Information], 0
        ; VOID IoCompleteRequest()
        invoke  IoCompleteRequest, [lpIrp], IO_NO_INCREMENT

        mov     eax, STATUS_SUCCESS
   .return:
        ret
endp

; ------------------------------------------------------------------------------

;  NTSTATUS
;  DispatchReadWrite (
;    IN PDEVICE_OBJECT  lpDeviceObject,
;    IN PIRP            lpIrp
;  )

align 10h
proc DispatchReadWrite, lpDeviceObject, lpIrp

        mov     [lpDeviceObject], rcx
        mov     [lpIrp], rdx
;       mov     [], r8
;       mov     [], r9

   .enter:
        test    rsp, 0x0F                       ; 16-byte stack align job
        jnz     @F
        push    rax
      @@:
        push    rbx rsi rdi

        mov     ebx, STATUS_UNSUCCESSFUL                                ; EBX = current status

        mov     rdi, [lpIrp]                                            ; RDI = PIRP
        mov     [rdi + V_IRP.IoStatus.Information], 0
        mov     rsi, [rdi + V_IRP.Tail.Overlay.CurrentStackLocation]    ; RSI = PIO_STACK_LOCATION

        mov     ecx, [rsi + V_IO_STACK_LOCATION.Parameters.Write.Length]
        cmp     ecx, RZ_DRIVER_BUFFER_SIZE
        ja     .leave

   .RZDriverQuery:
        mov     ebx, STATUS_NOT_IMPLEMENTED
        mov     rsi, [rdi + V_IRP.UserBuffer]                           ; RSI = PR0DriverQuery
        mov     qword [rsi + V_RZDriverQuery.RESULT], 0
        mov     ecx, dword [rsi + V_RZDriverQuery.IOCODE]               ; ECX = user I/O code

   .TEST.procs:
        cmp     ecx, RZ_DRIVER_QUERY_PROC_NOARGS
        je     .proc.noargs
        cmp     ecx, RZ_DRIVER_QUERY_PROC_NOARGS_IFRAME
        je     .proc.noargs.iframe
        cmp     ecx, RZ_DRIVER_QUERY_PROC_REGISTER
        je     .proc.register
        cmp     ecx, RZ_DRIVER_QUERY_PROC_REGISTER_IFRAME
        je     .proc.register.iframe
        cmp     ecx, RZ_DRIVER_QUERY_PROC_REGISTER_2P
        je     .proc.register.2p
        cmp     ecx, RZ_DRIVER_QUERY_PROC_REGISTER_2P_IFRAME
        je     .proc.register.2p.iframe
        cmp     ecx, RZ_DRIVER_QUERY_PROC_FASTCALL
        je     .proc.fastcall
        cmp     ecx, RZ_DRIVER_QUERY_PROC_FASTCALL_IFRAME
        je     .proc.fastcall.iframe
        cmp     ecx, RZ_DRIVER_QUERY_PROC_FASTCALL_2P
        je     .proc.fastcall.2p
        cmp     ecx, RZ_DRIVER_QUERY_PROC_FASTCALL_2P_IFRAME
        je     .proc.fastcall.2p.iframe
   .TEST.IO:
        cmp     ecx, RZ_DRIVER_QUERY_PORT_IN_BYTE
        je     .port.in.byte
        cmp     ecx, RZ_DRIVER_QUERY_PORT_IN_WORD
        je     .port.in.word
        cmp     ecx, RZ_DRIVER_QUERY_PORT_IN_DWORD
        je     .port.in.dword
        cmp     ecx, RZ_DRIVER_QUERY_PORT_OUT_BYTE
        je     .port.out.byte
        cmp     ecx, RZ_DRIVER_QUERY_PORT_OUT_WORD
        je     .port.out.word
        cmp     ecx, RZ_DRIVER_QUERY_PORT_OUT_DWORD
        je     .port.out.dword
   .TEST.map:
        cmp     ecx, RZ_DRIVER_QUERY_MAP_IO_SPACE
        je     .map.io.space
        cmp     ecx, RZ_DRIVER_QUERY_MAP_IO_SPACE_IFRAME
        je     .map.io.space.iframe
        cmp     ecx, RZ_DRIVER_QUERY_UNMAP_IO_SPACE
        je     .unmap.io.space
        cmp     ecx, RZ_DRIVER_QUERY_UNMAP_IO_SPACE_IFRAME
        je     .unmap.io.space.iframe
   .TEST.cp:
        cmp     ecx, RZ_DRIVER_QUERY_CP_SET
        je     .cp.set
        cmp     ecx, RZ_DRIVER_QUERY_CP_SET_IFRAME
        je     .cp.set.iframe
        cmp     ecx, RZ_DRIVER_QUERY_CP_RET
        je     .cp.ret
        cmp     ecx, RZ_DRIVER_QUERY_CP_RET_IFRAME
        je     .cp.ret.iframe
        cmp     ecx, RZ_DRIVER_QUERY_CP_GET
        je     .cp.get
        cmp     ecx, RZ_DRIVER_QUERY_CP_GET_IFRAME
        je     .cp.get.iframe

        jmp    .leave

   .proc.noargs:
        mov     rax, qword [rsi + V_RZDriverQuery.PROC]
        or      rax, rax
        jz     .leave
        sub     rsp, 32
        call    rax
        add     rsp, 32
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .proc.noargs.iframe:
        mov     rax, qword [rsi + V_RZDriverQuery.PROC]
        or      rax, rax
        jz     .leave
        cli
        sub     rsp, 32
        call    rax
        add     rsp, 32
        mov     ebx, STATUS_SUCCESS
        sti
        jmp    .leave

   .proc.register:
        mov     rcx, qword [rsi + V_RZDriverQuery.PROC]
        or      rcx, rcx
        jz     .leave
        sub     rsp, 32
        mov     rax, qword [rsi + V_RZDriverQuery.paramA]
        call    rcx
        add     rsp, 32
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .proc.register.iframe:
        mov     rcx, qword [rsi + V_RZDriverQuery.PROC]
        or      rcx, rcx
        jz     .leave
        cli
        sub     rsp, 32
        mov     rax, qword [rsi + V_RZDriverQuery.paramA]
        call    rcx
        add     rsp, 32
        mov     ebx, STATUS_SUCCESS
        sti
        jmp    .leave

   .proc.register.2p:
        mov     rcx, qword [rsi + V_RZDriverQuery.PROC]
        or      rcx, rcx
        jz     .leave
        sub     rsp, 32
        mov     rax, qword [rsi + V_RZDriverQuery.paramA]
        mov     rdx, qword [rsi + V_RZDriverQuery.paramB]
        call    rcx
        add     rsp, 32
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .proc.register.2p.iframe:
        mov     rcx, qword [rsi + V_RZDriverQuery.PROC]
        or      rcx, rcx
        jz     .leave
        cli
        sub     rsp, 32
        mov     rax, qword [rsi + V_RZDriverQuery.paramA]
        mov     rdx, qword [rsi + V_RZDriverQuery.paramB]
        call    rcx
        add     rsp, 32
        mov     ebx, STATUS_SUCCESS
        sti
        jmp    .leave

   .proc.fastcall:
        mov     rax, qword [rsi + V_RZDriverQuery.PROC]
        or      rax, rax
        jz     .leave
        sub     rsp, 32
        mov     rcx, qword [rsi + V_RZDriverQuery.paramA]
        call    rax
        add     rsp, 32
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .proc.fastcall.iframe:
        mov     rax, qword [rsi + V_RZDriverQuery.PROC]
        or      rax, rax
        jz     .leave
        cli
        sub     rsp, 32
        mov     rcx, qword [rsi + V_RZDriverQuery.paramA]
        call    rax
        add     rsp, 32
        mov     ebx, STATUS_SUCCESS
        sti
        jmp    .leave

   .proc.fastcall.2p:
        mov     rax, qword [rsi + V_RZDriverQuery.PROC]
        or      rax, rax
        jz     .leave
        sub     rsp, 32
        mov     rcx, qword [rsi + V_RZDriverQuery.paramA]
        mov     rdx, qword [rsi + V_RZDriverQuery.paramB]
        call    rax
        add     rsp, 32
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .proc.fastcall.2p.iframe:
        mov     rax, qword [rsi + V_RZDriverQuery.PROC]
        or      rax, rax
        jz     .leave
        cli
        sub     rsp, 32
        mov     rcx, qword [rsi + V_RZDriverQuery.paramA]
        mov     rdx, qword [rsi + V_RZDriverQuery.paramB]
        call    rax
        add     rsp, 32
        mov     ebx, STATUS_SUCCESS
        sti
        jmp    .leave

   .port.in.byte:
        mov     dx, word [rsi + V_RZDriverQuery.paramA]
        in      al, dx
        mov     byte [rsi + V_RZDriverQuery.RESULT], al
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .port.in.word:
        mov     dx, word [rsi + V_RZDriverQuery.paramA]
        in      ax, dx
        mov     word [rsi + V_RZDriverQuery.RESULT], ax
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .port.in.dword:
        mov     dx, word [rsi + V_RZDriverQuery.paramA]
        in      eax, dx
        mov     dword [rsi + V_RZDriverQuery.RESULT], eax
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .port.out.byte:
        mov     dx, word [rsi + V_RZDriverQuery.paramA]
        mov     al, byte [rsi + V_RZDriverQuery.paramB]
        out     dx, al
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .port.out.word:
        mov     dx, word [rsi + V_RZDriverQuery.paramA]
        mov     ax, word [rsi + V_RZDriverQuery.paramB]
        out     dx, ax
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .port.out.dword:
        mov     dx, word [rsi + V_RZDriverQuery.paramA]
        mov     eax, dword [rsi + V_RZDriverQuery.paramB]
        out     dx, eax
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .map.io.space:
        procall mapiospace, rsi
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .map.io.space.iframe:
        cli
        procall mapiospace, rsi
        mov     ebx, STATUS_SUCCESS
        sti
        jmp    .leave

   .unmap.io.space:
        procall unmapiospace, rsi
        mov     ebx, STATUS_SUCCESS
        jmp    .leave

   .unmap.io.space.iframe:
        cli
        procall unmapiospace, rsi
        mov     ebx, STATUS_SUCCESS
        sti
        jmp    .leave

   .cp.set:
        procall CreateProcessSet, rsi
        jmp    .leave

   .cp.set.iframe:
        cli
        procall CreateProcessSet, rsi
        sti
        jmp    .leave

   .cp.ret:
        procall CreateProcessRet, rsi
        jmp    .leave

   .cp.ret.iframe:
        cli
        procall CreateProcessRet, rsi
        sti
        jmp    .leave

   .cp.get:
        procall CreateProcessGet, rsi
        jmp    .leave

   .cp.get.iframe:
        cli
        procall CreateProcessGet, rsi
        sti
        jmp    .leave

   .leave:
        mov     [rdi + V_IRP.IoStatus.Status], ebx
        ; VOID IoCompleteRequest()
        invoke  IoCompleteRequest, rdi, IO_NO_INCREMENT

   .return:
        mov     eax, ebx
        pop     rdi rsi rbx
        ret
endp

; ------------------------------------------------------------------------------

; PVOID
; MmMapIoSpace(
;   IN PHYSICAL_ADDRESS     PhysicalAddress,
;   IN SIZE_T               NumberOfBytes,
;   IN MEMORY_CACHING_TYPE  CacheType
; );

; VOID
; MmUnmapIoSpace(
;   IN PVOID   BaseAddress,
;   IN SIZE_T  NumberOfBytes
; );

align 10h
proc mapiospace, lpQuery

;       mov     [lpQuery], rcx
;       mov     [], rdx
;       mov     [], r8
;       mov     [], r9

   .enter:
        test    rsp, 0x0F                       ; 16-byte stack align job
        jz      @F
        push    rax
      @@:
        push    rsi rdi

        mov     rsi, rcx ; lpQuery
        or      rsi, rsi
        jz     .return

        mov     rcx, qword [rsi + V_RZDriverIOQuery.SRC]        ; PhysicalAddress
        or      rcx, rcx
        jz     .return
        mov     rdx, qword [rsi + V_RZDriverIOQuery.BYTES]      ; NumberOfBytes
        or      rdx, rdx
        jz     .return
        xor     r8, r8                                          ; CacheType :: MmNonCached (FALSE)

        invoke  MmMapIoSpace, rcx, rdx, r8
        mov     qword [rsi + V_RZDriverIOQuery.RESULT], rax     ; Save virtual address
        or      rax, rax
        jz     .return

        mov     rcx, qword [rsi + V_RZDriverIOQuery.DST]        ; Destination
        mov     rdx, qword [rsi + V_RZDriverIOQuery.RESULT]     ; Source
        mov     r8, qword [rsi + V_RZDriverIOQuery.BYTES]       ; Length
        cinvoke memcpy, rcx, rdx, r8

        mov     ecx, dword [rsi + V_RZDriverIOQuery.IODATA]     ; Skip MmUnmapIoSpace if IODATA != 0
        or      ecx, ecx
        jnz    .return

        mov     rcx, qword [rsi + V_RZDriverIOQuery.RESULT]     ; BaseAddress
        mov     rdx, qword [rsi + V_RZDriverIOQuery.BYTES]      ; NumberOfBytes
        invoke  MmUnmapIoSpace, rcx, rdx

   .return:
        pop     rdi rsi
        ret
endp

; ------------------------------------------------------------------------------

; VOID
; MmUnmapIoSpace(
;   IN PVOID   BaseAddress,
;   IN SIZE_T  NumberOfBytes
; );

align 10h
proc unmapiospace, lpQuery

;       mov     [lpQuery], rcx
;       mov     [], rdx
;       mov     [], r8
;       mov     [], r9

   .enter:
        test    rsp, 0x0F                       ; 16-byte stack align job
        jz      @F
        push    rax
      @@:
        push    rsi rdi

        mov     rsi, rcx ; lpQuery
        or      rsi, rsi
        jz     .return

        mov     rcx, qword [rsi + V_RZDriverIOQuery.SRC]        ; BaseAddress
        or      rcx, rcx
        jz     .return
        mov     rdx, qword [rsi + V_RZDriverIOQuery.BYTES]      ; NumberOfBytes
        or      rdx, rdx
        jz     .return
        invoke  MmUnmapIoSpace, rcx, rdx

   .return:
        pop     rdi rsi
        ret
endp

; ------------------------------------------------------------------------------

align 10h
proc CreateProcessSet, lpQuery

;       mov     [lpQuery], rcx
;       mov     [], rdx
;       mov     [], r8
;       mov     [], r9

   .enter:
        test    rsp, 0x0F                       ; 16-byte stack align job
        jz      @F
        push    rax
      @@:

;       mov     rcx, [lpQuery]
        test    rcx, rcx
        jz     .return

        mov     rax, qword [rcx + V_RZDriverCPQuery.ProcessID]
        test    rax, rax
        jz      @F
        mov     qword [CProcessID], rax
    @@:

        cmp     [CP_Activated], 1
        je     .return

        invoke  PsSetCreateProcessNotifyRoutine, CreateProcessCallback, FALSE
        cmp     eax, STATUS_SUCCESS
        jne    .return ; break on error

        mov     [CP_Activated], 1
        jmp    .return

   .return:
        ret
endp

; ------------------------------------------------------------------------------

align 10h
proc CreateProcessRet, lpQuery

;       mov     [lpQuery], rcx
;       mov     [], rdx
;       mov     [], r8
;       mov     [], r9

   .enter:
        test    rsp, 0x0F                       ; 16-byte stack align job
        jz      @F
        push    rax
      @@:

        cmp     [CP_Activated], 0
        je     .return

        invoke  PsSetCreateProcessNotifyRoutine, CreateProcessCallback, TRUE
        cmp     eax, STATUS_SUCCESS
        jne    .return ; break on error

        mov     [CP_Activated], 0
        jmp    .return

   .return:
        ret
endp

; ------------------------------------------------------------------------------

align 10h
proc CreateProcessGet, lpQuery

;       mov     [lpQuery], rcx
;       mov     [], rdx
;       mov     [], r8
;       mov     [], r9

   .enter:
        test    rsp, 0x0F                       ; 16-byte stack align job
        jnz     @F
        push    rax
      @@:
        push    rbx rsi rdi

        mov     rdi, rcx ; lpQuery
        or      rdi, rdi
        jz     .return

        mov     rsi, [DeviceExtension]
        or      rsi, rsi
        jz     .return

        invoke  KeWaitForSingleObject, [KCPGEvent], Executive, KernelMode, FALSE, NULL
        invoke  KeWaitForSingleObject, [KCPCEvent], Executive, KernelMode, FALSE, NULL
   .lock:
        invoke  KeClearEvent, [KCPGEvent]
        xor     rcx, rcx
        mov     cx, [CP_BuffDepth]
        or      cx, cx
        jz     .unlock
   .copy:
        mov     rax, qword [rsi + V_RZDriverCPQuery.ParentID]
        mov     qword [rdi + V_RZDriverCPQuery.ParentID], rax
        mov     rax, qword [rsi + V_RZDriverCPQuery.ProcessID]
        mov     qword [rdi + V_RZDriverCPQuery.ProcessID], rax
        mov     rax, qword [rsi + V_RZDriverCPQuery.Create]
        mov     qword [rdi + V_RZDriverCPQuery.Create], rax
        mov     rax, qword [rsi + V_RZDriverCPQuery.RESULT]
        mov     qword [rdi + V_RZDriverCPQuery.RESULT], rax
        mov     word [rdi + V_RZDriverCPQuery.RESULT+4], cx
        mov     ax, [CP_DropCount]
        mov     word [rdi + V_RZDriverCPQuery.RESULT+6], ax
   .shift:
        lea     rsi, [rsi + sizeof.RZDriverQuery]
        lea     rdi, [rdi + sizeof.RZDriverQuery]
        dec     [CP_BuffDepth]
        loop   .copy
   .unlock:
        invoke  KeSetEvent, [KCPGEvent], 0, FALSE

   .return:
        pop     rdi rsi rbx
        ret
endp

; ------------------------------------------------------------------------------

; VOID
; CreateProcessCallback (
;   IN HANDLE   hParentID,
;   IN HANDLE   hProcessID,
;   IN BOOLEAN  bCreate
; )

align 10h
proc CreateProcessCallback, hParentID, hProcessID, bCreate

        mov     [hParentID], rcx
        mov     [hProcessID], rdx
        mov     [bCreate], r8
;       mov     [], r9

   .enter:
        test    rsp, 0x0F                       ; 16-byte stack align job
        jnz     @F
        push    rax
      @@:
        push    rdi

        cmp     [CP_Activated], 0
        je     .return

        cmp     qword [CProcessID], 0
        je      @F
        cmp     qword [CProcessID], rdx
        je     .return
    @@:

        ; assign current values into device extension.
        ; user-mode apps will pick it up using DeviceIoControl calls.
        mov     rdi, [DeviceExtension]
        or      rdi, rdi
        jz     .return

;       invoke  KeWaitForSingleObject, [KCPCEvent], Executive, KernelMode, FALSE, NULL
        invoke  KeWaitForSingleObject, [KCPGEvent], Executive, KernelMode, FALSE, NULL
   .lock:
        invoke  KeClearEvent, [KCPCEvent]
        inc     [CP_RecvCount]
        xor     rax, rax
        mov     ax, [CP_BuffDepth]
        cmp     ax, RZ_DRIVER_BUFFER_CAPACITY
        jb     .assign
        inc     [CP_DropCount]
        jmp    .unlock
   .assign:
        inc     [CP_BuffDepth]
        mov     rcx, sizeof.RZDriverQuery
        mul     rcx
        lea     rdi, [rdi + rax]
        mov     rcx, [hParentID]
        mov     qword [rdi + V_RZDriverCPQuery.ParentID], rcx
        mov     rcx, [hProcessID]
        mov     qword [rdi + V_RZDriverCPQuery.ProcessID], rcx
        mov     rcx, [bCreate]
        mov     qword [rdi + V_RZDriverCPQuery.Create], rcx
        xor     rcx, rcx
        mov     ecx, [CP_RecvCount]
        mov     qword [rdi + V_RZDriverCPQuery.RESULT], rcx
   .unlock:
        invoke  KeSetEvent, [KCPCEvent], 0, FALSE

   .event:
        invoke  KeReadStateEvent, [KCPCEvent]
        or      eax, eax
        jz     .event.OFF
        ; signal the event thus the user-mode apps listening will be aware
        ; that something interesting has happened.
        invoke  KeSetEvent, [KProcessEvent], 0, FALSE
;       invoke  KeClearEvent, [KProcessEvent]
   .event.OFF:

   .return:
        pop     rdi
        ret
endp

; ------------------------------------------------------------------------------

align 10h
proc memcopy, Destination, Source, Length

;       mov     [Destination], rcx
;       mov     [Source], rdx
;       mov     [Length], r8
;       mov     [], r9

   .enter:
        push    rsi rdi

        cld
        mov     rdi, rcx
        mov     rsi, rdx

        mov     rcx, r8
        shr     rcx, 2
        rep     movsd

        mov     rcx, r8
        and     rcx, 3
        rep     movsb

   .return:
        pop     rdi rsi
        ret
endp

; ------------------------------------------------------------------------------
