
; FASM SOURCE FILE
; http://flatassembler.net

; Ring-0 Procedure Call & Direct Port I/O
; header

; written by @L.CHEMIST (Andrey A. Meshkov)
; http://maalchemist.narod.ru
; maalchemist@yandex.ru
; maalchemist@gmail.com

; ------------------------------------------------------------------------------

Executive = 0

; ------------------------------------------------------------------------------

RZ_DRIVER_BUFFER_CAPACITY               = 64
RZ_DRIVER_BUFFER_SIZE                   = RZ_DRIVER_BUFFER_CAPACITY * sizeof.RZDriverQuery
RZ_DRIVER_QUERY_BUFFER_SIZE             = 24

; ------------------------------------------------------------------------------

RZ_DRIVER_QUERY_EMPTY                   = 0x0000 ;
RZ_DRIVER_QUERY_NULL                    = 0x0000 ;

RZ_DRIVER_QUERY_PROC_NOARGS             = 0x0010 ; ...
RZ_DRIVER_QUERY_PROC_REGISTER           = 0x0011 ; paramA->eax(rax) ...
RZ_DRIVER_QUERY_PROC_FASTCALL           = 0x0012 ; paramA->ecx(rcx) ...
RZ_DRIVER_QUERY_PROC_STDCALL            = 0x0013 ; push paramA ...
RZ_DRIVER_QUERY_PROC_CDECL              = 0x0014 ; push paramA ...

RZ_DRIVER_QUERY_PROC_NOARGS_IFRAME      = 0x0110 ; cli ... sti
RZ_DRIVER_QUERY_PROC_REGISTER_IFRAME    = 0x0111 ; cli paramA->eax(rax) ... sti
RZ_DRIVER_QUERY_PROC_FASTCALL_IFRAME    = 0x0112 ; cli paramA->ecx(rcx) ... sti
RZ_DRIVER_QUERY_PROC_STDCALL_IFRAME     = 0x0113 ; cli push paramA ... sti
RZ_DRIVER_QUERY_PROC_CDECL_IFRAME       = 0x0114 ; cli push paramA ... sti

RZ_DRIVER_QUERY_PROC_REGISTER_2P        = 0x0015 ; paramA->eax(rax), paramB->edx(rdx) ...
RZ_DRIVER_QUERY_PROC_FASTCALL_2P        = 0x0016 ; paramA->ecx(rcx), paramB->edx(rdx) ...
RZ_DRIVER_QUERY_PROC_STDCALL_2P         = 0x0017 ; push paramB, push paramA ...
RZ_DRIVER_QUERY_PROC_CDECL_2P           = 0x0018 ; push paramB, push paramA ...

RZ_DRIVER_QUERY_PROC_REGISTER_2P_IFRAME = 0x0115 ; cli paramA->eax(rax), paramB->edx(rdx) ... sti
RZ_DRIVER_QUERY_PROC_FASTCALL_2P_IFRAME = 0x0116 ; cli paramA->ecx(rcx), paramB->edx(rdx) ... sti
RZ_DRIVER_QUERY_PROC_STDCALL_2P_IFRAME  = 0x0117 ; cli push paramB, push paramA ... sti
RZ_DRIVER_QUERY_PROC_CDECL_2P_IFRAME    = 0x0118 ; cli push paramB, push paramA ... sti

RZ_DRIVER_QUERY_PORT_IN_BYTE            = 0x0020 ;
RZ_DRIVER_QUERY_PORT_IN_WORD            = 0x0021 ;
RZ_DRIVER_QUERY_PORT_IN_DWORD           = 0x0022 ;

RZ_DRIVER_QUERY_PORT_OUT_BYTE           = 0x0030 ;
RZ_DRIVER_QUERY_PORT_OUT_WORD           = 0x0031 ;
RZ_DRIVER_QUERY_PORT_OUT_DWORD          = 0x0033 ;

; ------------------------------------------------------------------------------

; memory map

RZ_DRIVER_QUERY_MAP_IO_SPACE            = 0x0040 ; ...
RZ_DRIVER_QUERY_MAP_IO_SPACE_IFRAME     = 0x0041 ; cli ... sti
RZ_DRIVER_QUERY_UNMAP_IO_SPACE          = 0x0042 ; ...
RZ_DRIVER_QUERY_UNMAP_IO_SPACE_IFRAME   = 0x0043 ; cli ... sti

; MmMapIoSpace
; The MmMapIoSpace routine maps the given physical address range to nonpaged system space.
; PVOID
;   MmMapIoSpace(
;     IN PHYSICAL_ADDRESS     PhysicalAddress,
;     IN ULONG                NumberOfBytes,
;     IN MEMORY_CACHING_TYPE  CacheType
;     );

; MmUnmapIoSpace
; The MmUnmapIoSpace routine unmaps a specified range of physical addresses previously mapped by MmMapIoSpace.
; VOID
;   MmUnmapIoSpace(
;     IN PVOID   BaseAddress,
;     IN SIZE_T  NumberOfBytes
;     );

; typedef enum _MEMORY_CACHING_TYPE {
;   MmNonCached,
;   MmCached,
;   MmWriteCombined
; } MEMORY_CACHING_TYPE;

; ------------------------------------------------------------------------------

; create process

RZ_DRIVER_QUERY_CP_SET                  = 0x0050 ; ...
RZ_DRIVER_QUERY_CP_SET_IFRAME           = 0x0051 ; cli ... sti
RZ_DRIVER_QUERY_CP_RET                  = 0x0052 ; ...
RZ_DRIVER_QUERY_CP_RET_IFRAME           = 0x0053 ; cli ... sti
RZ_DRIVER_QUERY_CP_GET                  = 0x0054 ; ...
RZ_DRIVER_QUERY_CP_GET_IFRAME           = 0x0055 ; cli ... sti

; ------------------------------------------------------------------------------

; load image

RZ_DRIVER_QUERY_LI_SET                  = 0x0060 ; ...
RZ_DRIVER_QUERY_LI_SET_IFRAME           = 0x0061 ; cli ... sti
RZ_DRIVER_QUERY_LI_RET                  = 0x0062 ; ...
RZ_DRIVER_QUERY_LI_RET_IFRAME           = 0x0063 ; cli ... sti
RZ_DRIVER_QUERY_LI_GET                  = 0x0064 ; ...
RZ_DRIVER_QUERY_LI_GET_IFRAME           = 0x0065 ; cli ... sti

; ------------------------------------------------------------------------------

; structure for driver query
; base
struct RZDriverQuery
  IOCODE dd ? ; user I/O code
  IODATA dd ? ; user I/O data
  PROC   dq ? ; procedure offset
  paramA dq ? ; parameter A
  paramB dq ? ; parameter B
  RESULT dq ? ; result
  Buffer db RZ_DRIVER_QUERY_BUFFER_SIZE dup (?)
ends
TYPEDEF T_RZDriverQuery, RZDriverQuery
TYPEDEF P_RZDriverQuery, PVOID
VIRTDEF V_RZDriverQuery, RZDriverQuery

; ------------------------------------------------------------------------------

; structure for driver query
; I/O, memory map
struct RZDriverIOQuery
  IOCODE dd ? ; user I/O code
  IODATA dd ? ; user I/O data
  SRC    dq ? ; source address
  DST    dq ? ; destination address
  BYTES  dq ? ; memory size in bytes
  RESULT dq ? ; result
  Buffer db RZ_DRIVER_QUERY_BUFFER_SIZE dup (?)
ends
TYPEDEF T_RZDriverIOQuery, RZDriverIOQuery
TYPEDEF P_RZDriverIOQuery, PVOID
VIRTDEF V_RZDriverIOQuery, RZDriverIOQuery

; ------------------------------------------------------------------------------

; structure for driver query
; create process
struct RZDriverCPQuery
  IOCODE    dd ? ; user I/O code
  IODATA    dd ? ; user I/O data
  ParentID  dq ? ; parent process ID
  ProcessID dq ? ; process ID
  Create    dq ? ; create state
  RESULT    dq ? ; result
  Buffer    db RZ_DRIVER_QUERY_BUFFER_SIZE dup (?)
ends
TYPEDEF T_RZDriverCPQuery, RZDriverCPQuery
TYPEDEF P_RZDriverCPQuery, PVOID
VIRTDEF V_RZDriverCPQuery, RZDriverCPQuery

; ------------------------------------------------------------------------------

; structure for driver query
; load image
struct RZDriverLIQuery
  IOCODE    dd ? ; user I/O code
  IODATA    dd ? ; user I/O data
  ProcessID dq ? ; process ID
  Image     dq ? ; image name buffer address
  Reserved  dq ? ; reserved
  RESULT    dq ? ; result
  Buffer    db RZ_DRIVER_QUERY_BUFFER_SIZE dup (?)
ends
TYPEDEF T_RZDriverLIQuery, RZDriverLIQuery
TYPEDEF P_RZDriverLIQuery, PVOID
VIRTDEF V_RZDriverLIQuery, RZDriverLIQuery

; ------------------------------------------------------------------------------

; be sure sizeof.RZDriverQuery = sizeof.RZDriverIOQuery
if sizeof.RZDriverQuery <> sizeof.RZDriverIOQuery
   display 'sizeof.RZDriverQuery <> sizeof.RZDriverIOQuery',13,10
   oops
end if

; be sure sizeof.RZDriverQuery = sizeof.RZDriverCPQuery
if sizeof.RZDriverQuery <> sizeof.RZDriverCPQuery
   display 'sizeof.RZDriverQuery <> sizeof.RZDriverCPQuery',13,10
   oops
end if

; be sure sizeof.RZDriverQuery = sizeof.RZDriverLIQuery
if sizeof.RZDriverQuery <> sizeof.RZDriverLIQuery
   display 'sizeof.RZDriverQuery <> sizeof.RZDriverLIQuery',13,10
   oops
end if

; ------------------------------------------------------------------------------
